@@ -22,6 +22,7 @@
 #ifndef GR_UNIXGRAPHICS_H
 #define GR_UNIXGRAPHICS_H
 
+
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 #include "xap_UnixApp.h"
@@ -36,65 +37,73 @@ class UT_ByteBuf;
 class UT_String;
 class UT_Wctomb;
 
+class XAP_App;
+class XAP_UnixFontManager;
+
 UT_uint32 adobeToUnicode(UT_uint32 iAdobe);
 
 UT_uint32 adobeDingbatsToUnicode(UT_uint32 iAdobe);
 
+class GR_UnixAllocInfo : public GR_AllocInfo
+{
+public:
+ 	GR_UnixAllocInfo(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App *app)
+		: m_win(win), m_pixmap(NULL), m_fontManager(fontManager), m_usePixmap(false) {};
+
+	GR_UnixAllocInfo(GdkPixmap * win, XAP_UnixFontManager * fontManager, XAP_App *app, bool bUsePixmap)
+		: m_win(NULL), m_pixmap(win), m_fontManager(fontManager), m_usePixmap(bUsePixmap) {};
+
+	virtual GR_GraphicsId getType() const {return GRID_UNIX;};
+	virtual bool isPrinterGraphics() const {return false; };
+
+	GdkWindow* m_win;
+	GdkPixmap* m_pixmap;
+	XAP_UnixFontManager * m_fontManager;
+	bool m_usePixmap;
+};
+
 class GR_UnixGraphics : public GR_Graphics
 {
+	friend class GR_UnixImage;
+
+	// all constructors are protected; instances must be created via
+	// GR_GraphicsFactory
  public:
-#ifndef WITH_PANGO
- 	GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App *app);
-#else
- 	GR_UnixGraphics(GdkWindow * win, XAP_App *app);
-#endif
 	virtual ~GR_UnixGraphics();
 
-#ifndef WITH_PANGO 
-	virtual void        drawGlyph(UT_uint32 glyph_idx, UT_sint32 xoff, UT_sint32 yoff);
-	virtual void		drawChars(const UT_UCSChar* pChars, int iCharOffset,
-								  int iLength, UT_sint32 xoff, UT_sint32 yoff,
-								  int * pCharWidths = NULL);
+	static UT_uint32 s_getClassId() {return GRID_UNIX;}
+	virtual UT_uint32 getClassId() {return s_getClassId();}
+	
+	virtual GR_Capability getCapability(){UT_ASSERT(UT_NOT_IMPLEMENTED); return GRCAP_UNKNOWN;}
+	static const char *    graphicsDescriptor(void){return "Unix Default";}
+	static GR_Graphics *   graphicsAllocator(GR_AllocInfo& allocInfo);
+	
 	virtual void		setFont(GR_Font* pFont);
+	virtual void        clearFont(void) {m_pFont = NULL;} 
 	virtual UT_uint32	getFontHeight();
 	// virtual UT_uint32	measureString(const UT_UCSChar*s, int iOffset, int num, unsigned short* pWidths);
-	virtual UT_uint32 measureUnRemappedChar(const UT_UCSChar c);
-#endif	
+	virtual UT_sint32 measureUnRemappedChar(const UT_UCSChar c);
+
 	virtual void		setColor(const UT_RGBColor& clr);
 	virtual void        getColor(UT_RGBColor &clr);
 
 	virtual GR_Font*	getGUIFont();
 
-#ifndef WITH_PANGO 	
-	virtual GR_Font*	findFont(const char* pszFontFamily, 
-								 const char* pszFontStyle, 
-								 const char* pszFontVariant, 
-								 const char* pszFontWeight, 
-								 const char* pszFontStretch, 
-								 const char* pszFontSize);
 	virtual GR_Font*	getDefaultFont(UT_String& fontFamily);
 
 	virtual UT_uint32	getFontAscent();
 	virtual UT_uint32	getFontDescent();
 
-	virtual void		getCoverage(UT_Vector& coverage);
-#endif	
-	virtual void		drawLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
+	virtual void		getCoverage(UT_NumberVector& coverage);
+
 	virtual void		setLineWidth(UT_sint32);
-	virtual void		xorLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
-	virtual void		polyLine(UT_Point * pts, UT_uint32 nPoints);
-	virtual void		fillRect(const UT_RGBColor& c,
-								 UT_sint32 x, UT_sint32 y,
-								 UT_sint32 w, UT_sint32 h);
-	virtual void		invertRect(const UT_Rect* pRect);
+
 	virtual void		setClipRect(const UT_Rect* pRect);
 	virtual void		scroll(UT_sint32, UT_sint32);
 	virtual void		scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 							   UT_sint32 x_src, UT_sint32 y_src,
 							   UT_sint32 width, UT_sint32 height);
-	virtual void		clearArea(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
-  
-	virtual void		drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest);
+
 	virtual GR_Image*	createNewImage(const char* pszName, const UT_ByteBuf* pBB,
 					       UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight,
 					       GR_Image::GRType = GR_Image::GRT_Raster);
@@ -117,12 +126,6 @@ class GR_UnixGraphics : public GR_Graphics
 
 	virtual void		setColor3D(GR_Color3D c);
 	void				init3dColors(GtkStyle * pStyle);
-	virtual void		fillRect(GR_Color3D c,
-								 UT_sint32 x, UT_sint32 y,
-								 UT_sint32 w, UT_sint32 h);
-	virtual void		fillRect(GR_Color3D c, UT_Rect &r);
-	
-	virtual void		polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints);
 
 	void                createPixmapFromXPM( char ** pXPM,GdkPixmap *source,
 											 GdkBitmap * mask);
@@ -132,15 +135,11 @@ class GR_UnixGraphics : public GR_Graphics
 					 GR_Graphics::CapStyle inCapStyle   = CAP_BUTT,
 					 GR_Graphics::LineStyle inLineStyle = LINE_SOLID ) ;
 
-#ifndef WITH_PANGO 	
 	/* GR_Font versions of the above -- TODO: should I add drawChar* methods too? */
 	virtual UT_uint32 getFontAscent(GR_Font *);
 	virtual UT_uint32 getFontDescent(GR_Font *);
 	virtual UT_uint32 getFontHeight(GR_Font *);
-#else
- private:
-	virtual void _drawFT2Bitmap(UT_sint32 x, UT_sint32 y, FT_Bitmap * pBitmap) const;
-#endif
+    virtual GR_Image * genImageFromRectangle(const UT_Rect & r);
 	virtual void	  saveRectangle(UT_Rect & r, UT_uint32 iIndx);
 	virtual void	  restoreRectangle(UT_uint32 iIndx);
 	
@@ -148,19 +147,52 @@ class GR_UnixGraphics : public GR_Graphics
 	virtual UT_uint32 	getDeviceResolution(void) const;
 
  protected:
+	// all instances have to be created via GR_GraphicsFactory; see gr_Graphics.h
+ 	GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App *app);
+	GR_UnixGraphics(GdkPixmap * win, XAP_UnixFontManager * fontManager, XAP_App *app, bool bUsePixmap);
+
+	virtual void _beginPaint ();
+	virtual void _endPaint ();
+
+	virtual void        drawGlyph(UT_uint32 glyph_idx, UT_sint32 xoff, UT_sint32 yoff);
+	virtual void		drawChars(const UT_UCSChar* pChars, int iCharOffset,
+								  int iLength, UT_sint32 xoff, UT_sint32 yoff,
+								  int * pCharWidths = NULL);
+	virtual void		fillRect(GR_Color3D c,
+								 UT_sint32 x, UT_sint32 y,
+								 UT_sint32 w, UT_sint32 h);
+	virtual void		fillRect(GR_Color3D c, UT_Rect &r);   
+	virtual void		polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints);
+	virtual void		clearArea(UT_sint32, UT_sint32, UT_sint32, UT_sint32);  
+	virtual void		drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest);
+	virtual void		xorLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
+	virtual void		polyLine(UT_Point * pts, UT_uint32 nPoints);
+	virtual void		fillRect(const UT_RGBColor& c,
+								 UT_sint32 x, UT_sint32 y,
+								 UT_sint32 w, UT_sint32 h);
+	virtual void		invertRect(const UT_Rect* pRect);
+	virtual void		drawLine(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
+	bool                isDingbat(void) const;
+	bool                isSymbol(void) const;
+
+	virtual GR_Font*	_findFont(const char* pszFontFamily, 
+								  const char* pszFontStyle, 
+								  const char* pszFontVariant, 
+								  const char* pszFontWeight, 
+								  const char* pszFontStretch, 
+								  const char* pszFontSize);
+
 	void				_setColor(GdkColor & c);
-#ifndef WITH_PANGO
+
 	XAP_UnixFontManager * 	m_pFontManager;
-#endif	
 
 	GdkGC*       			m_pGC;
 	GdkGC*  	      		m_pXORGC;
 	GdkWindow*  	  		m_pWin;
 
-#ifndef WITH_PANGO	
 	// our currently requested font by handle
 	XAP_UnixFontHandle *	m_pFont;
-#endif
+
 	// Current GDK fonts corresponding to this. Calling m_pFont->explodeGdkFont
 	// causes gdk_font_load to be called and memory to be allocated. This should
 	// not happen on every draw
@@ -179,9 +211,7 @@ class GR_UnixGraphics : public GR_Graphics
 	
 	GdkColor				m_3dColors[COUNT_3D_COLORS];
 private:
-#ifndef WITH_PANGO 	
 	XAP_UnixFontHandle *	m_pFallBackFontHandle;
-#endif
 
 	XftDraw*				m_pXftDraw;
 	XftColor				m_XftColor;
@@ -197,8 +227,8 @@ private:
 	UT_sint32               m_iYoff;
 	UT_RGBColor				m_curColor;
 
-	UT_Vector				m_vSaveRect;
-	UT_Vector 				m_vSaveRectBuf;
+	UT_GenericVector<UT_Rect*>     m_vSaveRect;
+	UT_GenericVector<GdkPixbuf *>  m_vSaveRectBuf;
 
 	bool                    m_bIsSymbol;       
 	bool                    m_bIsDingbat;       
