@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Application Framework
  * Copyright (C) 1998 AbiSource, Inc.
  * Copyright (C) 2001-2003 Hubert Figuiere
@@ -17,7 +19,6 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  * 02111-1307, USA.
  */
- /* $Id: xap_CocoaDlg_FontChooser.h,v 1.7 2003/05/13 07:45:59 hub Exp $ */
 
 #ifndef XAP_COCOADIALOG_FONTCHOOSER_H
 #define XAP_COCOADIALOG_FONTCHOOSER_H
@@ -36,34 +37,39 @@ class XAP_CocoaDialog_FontChooser;
 
 @interface XAP_CocoaDialog_FontChooserController : NSWindowController <XAP_CocoaDialogProtocol>
 {
-	IBOutlet NSTextField*		_fontLabel;
-	IBOutlet NSTableView*		_fontList;
-	IBOutlet NSTextField*		_styleLabel;
-	IBOutlet NSTableView*		_styleList;
-	IBOutlet NSTextField*		_sizeLabel;
-	IBOutlet NSTableView*		_sizeList;
-	IBOutlet NSTextField*		_effectLabel;
-	IBOutlet NSButton*			_strikeButton;
-	IBOutlet NSButton*			_underlineButton;
-	IBOutlet NSButton*			_overlineButton;
-	IBOutlet NSButton*			_hiddenButton;
-	IBOutlet NSTextField*		_textColorLabel;
-	IBOutlet NSColorWell*		_textColorWell;
-	IBOutlet NSTextField*		_textHighlightColorLabel;
-	IBOutlet NSColorWell*		_textHighlightColorWell;
-	IBOutlet NSButton*			_noHighlightColorButton;
+	IBOutlet NSTableView *		_fontList;
+	IBOutlet NSTableView *		_styleList;
+	IBOutlet NSTableView *		_sizeList;
 
-	/* bottom part */
-	IBOutlet XAP_CocoaNSView * _preview;
-	IBOutlet NSButton*			_okBtn;
-	IBOutlet NSButton*			_cancelBtn;
-	IBOutlet NSButton*			_helpBtn;
-	IBOutlet NSBox*			_previewBox;
+	IBOutlet NSTextField *		_fontLabel;
+	IBOutlet NSTextField *		_styleLabel;
+	IBOutlet NSTextField *		_sizeLabel;
+	IBOutlet NSTextField *		_effectLabel;
+	IBOutlet NSTextField *		_textColorLabel;
+	IBOutlet NSTextField *		_textHighlightColorLabel;
+
+	IBOutlet NSButton *			_okBtn;
+	IBOutlet NSButton *			_cancelBtn;
+	IBOutlet NSButton *			_strikeButton;
+	IBOutlet NSButton *			_underlineButton;
+	IBOutlet NSButton *			_overlineButton;
+	IBOutlet NSButton *			_hiddenButton;
+	IBOutlet NSButton *			_superscriptButton;
+	IBOutlet NSButton *			_subscriptButton;
+	IBOutlet NSButton *			_noHighlightColorButton;
+
+	IBOutlet NSColorWell *		_textColorWell;
+	IBOutlet NSColorWell *		_textHighlightColorWell;
+
+	IBOutlet XAP_CocoaNSView *	_preview;
+
+	IBOutlet NSBox *			_previewBox;
 	
-	XAP_StringListDataSource*	m_fontDataSource;
-	XAP_StringListDataSource*	m_sizeDataSource;
-	XAP_StringListDataSource*	m_stylesDataSource;
-	XAP_CocoaDialog_FontChooser*	_xap;
+	XAP_StringListDataSource *	m_fontDataSource;
+	XAP_StringListDataSource *	m_sizeDataSource;
+	XAP_StringListDataSource *	m_stylesDataSource;
+
+	XAP_CocoaDialog_FontChooser *	_xap;
 }
 
 -(IBAction)okAction:(id)sender;
@@ -72,14 +78,19 @@ class XAP_CocoaDialog_FontChooser;
 -(IBAction)underlineAction:(id)sender;
 -(IBAction)overlineAction:(id)sender;
 -(IBAction)strikeoutAction:(id)sender;
+-(IBAction)superscriptAction:(id)sender;
+-(IBAction)subscriptAction:(id)sender;
 -(IBAction)transparentAction:(id)sender;		// = no higlight color ?
 
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
 
+- (XAP_CocoaNSView*)preview;
 /* accessors */
 -(void)setStrikeout:(bool)value;
 -(void)setUnderline:(bool)value;
 -(void)setOverline:(bool)value;
+-(void)setSuperscript:(bool)value;
+-(void)setSubscript:(bool)value;
 
 -(void)selectFont:(char*)value;
 -(NSString*)selectedFont;
@@ -108,6 +119,8 @@ public:
 	void                    underlineChanged(bool);
 	void                    overlineChanged(bool);
 	void                    strikeoutChanged(bool);
+	void                    superscriptChanged(bool);
+	void                    subscriptChanged(bool);
 	void                    transparencyChanged(bool);
 	void 					updatePreview(void);
 	void                    fontRowChanged(void);
@@ -129,9 +142,12 @@ public:
 	void				_deleteGC(void);
 	
 private:
+	bool		m_bSuperScriptInitialValue;
+	bool		m_bSubScriptInitialValue;
+
 	void _colorChanged(NSColor* color, const XML_Char* attr, char* buf);
 
-	XML_Char*	m_currentFamily;
+	XML_Char *	m_currentFamily;
 	XAP_CocoaDialog_FontChooserController*	m_dlg;
 };
 
