@@ -27,6 +27,7 @@
 #include "fl_BlockLayout.h"
 #include "fp_Run.h"
 #include "fp_Line.h"
+#include "fv_View.h"
 #include "pd_Document.h"
 #include "pt_PieceTable.h"
 #include "pf_Frag.h"
@@ -47,10 +48,12 @@ fl_AutoNum::fl_AutoNum(	UT_uint32 id,
 						fl_AutoNum * pParent,
 						const XML_Char * lDelim,
 						const XML_Char * lDecimal,
-						List_Type lType,
-						PD_Document * pDoc)
+						FL_ListType lType,
+						PD_Document * pDoc,
+						FV_View * pView)
 	:	m_pParent(pParent),
 		m_pDoc(pDoc),
+		m_pView(pView),
 		m_List_Type(lType),
 		m_iID(id),
 		m_iParentID(0),
@@ -78,13 +81,15 @@ fl_AutoNum::fl_AutoNum(	UT_uint32 id,
 
 fl_AutoNum::fl_AutoNum(	UT_uint32 id,
 						UT_uint32 parent_id,
-						List_Type lType,
+						FL_ListType lType,
 						UT_uint32 start,
 						const XML_Char * lDelim,
 						const XML_Char * lDecimal,
-						PD_Document * pDoc)
+						PD_Document * pDoc,
+						FV_View * pView)
 	:	m_pParent(0),
 		m_pDoc(pDoc),
+		m_pView(pView),
 		m_List_Type(lType),
 		m_iID(id),
 		m_iParentID(parent_id),
@@ -105,11 +110,21 @@ fl_AutoNum::fl_AutoNum(	UT_uint32 id,
 	if(m_iParentID != 0)
 	{
 		_setParent(m_pDoc->getListByID(parent_id));
-		if(m_pParent != NULL)
-			m_iLevel = m_pParent->getLevel() + 1;
 	}
 }
 
+bool fl_AutoNum::checkReference(fl_AutoNum * pAuto)
+{
+	if(pAuto == m_pParent)
+	{
+		return false;
+	}
+	if(m_pParent)
+	{
+		return m_pParent->checkReference(pAuto);
+	}
+	return true;
+}
 
 void fl_AutoNum::addItem(PL_StruxDocHandle pItem)
 {
@@ -134,7 +149,10 @@ void fl_AutoNum::fixHierarchy(void)
 	if(m_pItems.getItemCount() >0)
 	{
 		PL_StruxDocHandle sdh = static_cast<PL_StruxDocHandle>(m_pItems.getNthItem(0));
-		bool bFound = m_pDoc->getAttributeFromSDH(sdh,PT_PARENTID_ATTRIBUTE_NAME,&pszParentID);
+		bool bFound = m_pDoc->getAttributeFromSDH(sdh,
+												  m_pView ? m_pView->isShowRevisions() : true,
+												  m_pView ? m_pView->getRevisionLevel(): PD_MAX_REVISION,
+												  PT_PARENTID_ATTRIBUTE_NAME,&pszParentID);
 		if(bFound)
 		{
 			docParentID= atoi(pszParentID);
@@ -385,7 +403,7 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 		i++;
 	}
 	if (i >= NrElements(p) || p[i] == '\0') {
-		UT_ASSERT(UT_NOT_REACHED);
+//		UT_ASSERT(UT_NOT_REACHED);
 		UT_DEBUGMSG (("Hub: not a delim (SHOULD NOT HAPPEN)!!!\n"));
 		*insPoint = 0;
 		return;
@@ -559,9 +577,8 @@ void    fl_AutoNum::_getLabelstr( UT_UCSChar labelStr[], UT_uint32 * insPoint,
 const UT_UCSChar * fl_AutoNum::getLabel(PL_StruxDocHandle pItem)  const
 {
 	static UT_UCSChar label[100];
-	UT_uint32  insPoint=0;
-	UT_uint32 depth;
-	depth = 0;
+	UT_uint32 insPoint = 0;
+	UT_uint32 depth = 0;
 	_getLabelstr( label, &insPoint, depth , pItem);
 	if(insPoint == 0 )
 	{
@@ -579,7 +596,7 @@ UT_uint32 fl_AutoNum::getValue(PL_StruxDocHandle pItem) const
 }
 
 
-void fl_AutoNum::setListType(List_Type lType)
+void fl_AutoNum::setListType(FL_ListType lType)
 {
 	m_List_Type = lType;
 }
@@ -614,7 +631,7 @@ void fl_AutoNum::setDecimal(const XML_Char * lDecimal)
 	m_bDirty = true;
 }
 
-List_Type fl_AutoNum::getType() const
+FL_ListType fl_AutoNum::getType() const
 {
 	return m_List_Type;
 }
@@ -942,7 +959,7 @@ bool fl_AutoNum::doesItemHaveLabel( fl_BlockLayout * pItem)
 				return true;
 			}
 		}
-		pRun = pRun->getNext();
+		pRun = pRun->getNextRun();
 		if(pRun == NULL)
 		{
 			bStop = true;
@@ -974,6 +991,24 @@ fl_AutoNum * fl_AutoNum::getActiveParent(void)
 	return pAutoNum;
 }
 
+/*!
+ * This method returns true if the requested ID is somewhere in this
+ * List heiracy.
+ */
+bool fl_AutoNum::isIDSomeWhere(UT_uint32 ID)
+{
+	fl_AutoNum * pAuto = this;
+	while(pAuto != NULL)
+	{
+		if(pAuto->getID() == ID)
+		{
+			return true;
+		}
+		pAuto = pAuto->getParent();
+	}
+	return false;
+}
+
 void fl_AutoNum::_setParent(fl_AutoNum * pParent)
 {
 	if(pParent == this)
@@ -989,6 +1024,13 @@ void fl_AutoNum::_setParent(fl_AutoNum * pParent)
 		m_pParent = pParent;
 		if(m_pParent != NULL)
 		{
+			if(!pParent->checkReference(this))
+			{
+				m_pParent = NULL;
+				m_iParentID = 0;
+				m_bDirty = true;
+				return;
+			}
 			m_iParentID  = pParent->getID();
 		}
 		else
