@@ -38,10 +38,11 @@
 #include "fg_Graphic.h"
 #include "fl_AutoLists.h"
 #include "pp_Property.h"
-#include <fribidi.h>
 #include "fl_ContainerLayout.h"
 #include "fl_SectionLayout.h"
 #include "fb_LineBreaker.h"
+#include "ut_string_class.h"
+
 // number of DocPositions occupied by the block strux
 #define fl_BLOCK_STRUX_OFFSET	1
 
@@ -65,6 +66,7 @@ class PX_ChangeRecord_SpanChange;
 class PX_ChangeRecord_Strux;
 class PX_ChangeRecord_StruxChange;
 class fl_AutoNum;
+class fp_VerticalContainer;
 
 // Tab types and leaders
 typedef enum {
@@ -88,65 +90,20 @@ typedef enum {
 } eTabLeader;
 
 
-class ABI_EXPORT fl_CharWidths
-{
-public:
-	fl_CharWidths() : m_gbCharWidths(256)
-		{
-		}
-
-private:
-
-	UT_GrowBuf m_gbCharWidths;
-
-public:
-
-	bool ins(UT_uint32 position, UT_uint32 length)
-		{
-			return m_gbCharWidths.ins(position, length);
-		}
-
-	bool del(UT_uint32 position, UT_uint32 amount)
-		{
-			return m_gbCharWidths.del(position, amount);
-		}
-	UT_uint32 getLength(void) const
-		{
-			return m_gbCharWidths.getLength();
-		}
-	bool ins(UT_uint32 position, const UT_GrowBufElement *pValue,UT_uint32 length)
-		{
-			return m_gbCharWidths.ins(position, pValue, length);
-		}
-	bool ins(UT_uint32 position, const fl_CharWidths &Other, UT_uint32 offset, UT_uint32 length)
-		{
-			return m_gbCharWidths.ins(position, Other.m_gbCharWidths.getPointer(offset), length);
-		}
-
-	void truncate(UT_uint32 position)
-		{
-			m_gbCharWidths.truncate(position);
-		}
-
-	UT_GrowBuf *getCharWidths()
-		{
-			return &m_gbCharWidths;
-		}
-};
-
-
 /*
 	Blocks are stored in a linked list which contains all of the blocks in
 	the normal flow, in order.
 */
 
+class SpellChecker;
 class fl_TabStop;
-void buildTabStops(GR_Graphics * pG, const char* pszTabStops, UT_Vector &m_vecTabs);
+void buildTabStops(GR_Graphics * pG, const char* pszTabStops, UT_GenericVector<fl_TabStop*> &m_vecTabs);
 
 class ABI_EXPORT fl_BlockLayout : public fl_ContainerLayout
 {
 	friend class fl_Squiggles;
 	friend class fl_DocListener;
+	friend class fl_TOCLayout;
 	friend class fb_LineBreaker;
 
 	// TODO: shack - code should be moved from toggleAuto to a function in
@@ -155,7 +112,7 @@ class ABI_EXPORT fl_BlockLayout : public fl_ContainerLayout
 
 public:
 	fl_BlockLayout(PL_StruxDocHandle sdh,
-				   fl_BlockLayout*, fl_SectionLayout*,
+				   fl_ContainerLayout* pPrev, fl_SectionLayout*,
 				   PT_AttrPropIndex indexAP, bool bIsHdrFtr = false);
 	~fl_BlockLayout();
 
@@ -166,54 +123,62 @@ public:
 		spacing_ATLEAST
 	} eSpacingPolicy;
 
+    void                formatAll(void);
 	virtual void        format(void);
+	void                formatWrappedFromHere(fp_Line * pLine,fp_Page * pPage);
+	fp_Line *           getNextWrappedLine(UT_sint32 iX,
+											  UT_sint32 iHeight,
+										   fp_Page * pPage);
 	virtual bool		recalculateFields(UT_uint32 iUpdateCount);
 
 	virtual void		redrawUpdate();
 	virtual void        updateLayout(void) {}
 	virtual fp_Container * getNewContainer(fp_Container * pCon = NULL);
-	FV_View *		getView(void) const { return m_pLayout->getView(); }
+	FV_View *		getView(void) const { UT_return_val_if_fail( m_pLayout, NULL ); return m_pLayout->getView(); }
 
 	const char* getProperty(const XML_Char * pszName, bool bExpandStyles = true) const;
 	const PP_PropertyType * getPropertyType(const XML_Char * szName, tProperty_type Type, bool bExpandStyles = true) const;
 	void setAlignment(UT_uint32 iAlignCmd);
 	UT_sint32       getLength(void);
 	bool            isEmbeddedType(void);
-	void            updateOffsets(PT_DocPosition posEmbedded, UT_uint32 iEmebbedSize);
+	bool            isNotTOCable(void);
+	void            updateOffsets(PT_DocPosition posEmbedded, UT_uint32 iEmebbedSize, UT_sint32 iSuggestedDiff);
 	void            updateEnclosingBlockIfNeeded(void);
 	UT_sint32       getEmbeddedOffset(UT_sint32 startOffset, fl_ContainerLayout *& pEmbedCL);
 	void            shuffleEmbeddedIfNeeded(fl_BlockLayout * pBlock, UT_uint32 blockOffset);
 
+	bool            getXYOffsetToLine(UT_sint32 & xoff, UT_sint32 & yoff, fp_Line * pLine);
+	bool            setFramesOnPage(fp_Line * pLastLine);
 	UT_sint32       getHeightOfBlock(void);
 	fp_Line *       findLineWithFootnotePID(UT_uint32 pid);
 	UT_sint32 getMaxNonBreakableRun(void);
-
 	fp_Line* findPrevLineInDocument(fp_Line*);
 	fp_Line* findNextLineInDocument(fp_Line*);
 	virtual void     appendTextToBuf(UT_GrowBuf & buf);
+	void             appendUTF8String(UT_UTF8String & sText);
 	virtual fp_Run* getFirstRun(void) const { return m_pFirstRun; }
 	inline void setFirstRun(fp_Run* pRun) { m_pFirstRun = pRun; }
 
 	inline bool isListItem(void) const { return m_bListItem; }
 	bool isFirstInList(void);
 //	inline fl_AutoNum * getAutoNum(void) const { return m_pAutoNum; }
-	void	getListAttributesVector( UT_Vector * va);
-	void  getListPropertyVector( UT_Vector * vp);
+	void	getListAttributesVector(UT_GenericVector<const XML_Char*> * va);
+	void  getListPropertyVector(UT_GenericVector<const XML_Char*> * vp);
 
-	char *	getFormatFromListType(List_Type iListType);
+	char *	getFormatFromListType(FL_ListType iListType);
 	void remItemFromList(void);
 	virtual void listUpdate(void);
 	void resumeList( fl_BlockLayout * prevList);
 	void prependList( fl_BlockLayout * nextList);
-	List_Type decodeListType(char * listformat);
-	List_Type getListType(void);
-	XML_Char* getListStyleString( List_Type iListType);
-	List_Type getListTypeFromStyle( const XML_Char * style);
+	FL_ListType decodeListType(char * listformat);
+	FL_ListType getListType(void);
+	XML_Char* getListStyleString( FL_ListType iListType);
+	FL_ListType getListTypeFromStyle( const XML_Char * style);
 	fl_BlockLayout * getNextList(UT_uint32 id);
 	bool isListLabelInBlock(void);
 	void StartList( const XML_Char * style, PL_StruxDocHandle prevSDH = NULL);
 
-	void StartList( List_Type lType, UT_uint32 start,const XML_Char * lDelim, const XML_Char * lDecimal, const XML_Char * fFont, float Align, float indent, UT_uint32 iParentID = 0, UT_uint32 level=0 );
+	void StartList( FL_ListType lType, UT_uint32 start,const XML_Char * lDelim, const XML_Char * lDecimal, const XML_Char * fFont, float Align, float indent, UT_uint32 iParentID = 0, UT_uint32 level=0 );
 
 	void StopListInBlock(void);
 	void deleteListLabel(void);
@@ -230,18 +195,16 @@ public:
 	void findSquigglesForRun(fp_Run* pRun);
 	UT_uint32 canSlurp(fp_Line* pLine) const;
 
-	fl_CharWidths * getCharWidths(void);
-
 	PT_DocPosition getPosition(bool bActualBlockPos=false) const;
 	fp_Run* findPointCoords(PT_DocPosition position, bool bEOL, UT_sint32& x, UT_sint32& y, UT_sint32& x2, UT_sint32& y2, UT_sint32& height, bool& bDirection);
 
 	fp_Run* findRunAtOffset(UT_uint32 offset) const;
 	
-	bool getSpanPtr(UT_uint32 offset, const UT_UCSChar ** ppSpan, UT_uint32 * pLength) const;
 	bool	getBlockBuf(UT_GrowBuf * pgb) const;
 
 	void clearScreen(GR_Graphics*);
 
+	void                getStyle(UT_UTF8String & sStyle);
 	inline UT_sint32	getTextIndent(void) const { return m_iTextIndent; }
 	inline UT_sint32	getLeftMargin(void) const { return m_iLeftMargin; }
 	inline UT_sint32	getRightMargin(void) const { return m_iRightMargin; }
@@ -257,15 +220,13 @@ public:
 	void getLineSpacing(double& dSpacing,
 						eSpacingPolicy& eSpacing) const;
 
-	virtual void updateBackgroundColor(void);
-
 	inline UT_uint32 getProp_Orphans(void) const { return m_iOrphansProperty; }
 	inline UT_uint32 getProp_Widows(void) const { return m_iWidowsProperty; }
 	inline bool getProp_KeepTogether(void) const { return m_bKeepTogether; }
 	inline bool getProp_KeepWithNext(void) const { return m_bKeepWithNext; }
 
-	inline FriBidiCharType getDominantDirection(void) const { return m_iDomDirection; }
-	void setDominantDirection(FriBidiCharType iDirection);
+	inline UT_BidiCharType getDominantDirection(void) const { return m_iDomDirection; }
+	void setDominantDirection(UT_BidiCharType iDirection);
 
 	inline fl_Squiggles* getSquiggles(void) const { return m_pSquiggles; }
 
@@ -321,6 +282,13 @@ public:
 								   void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
 														   PL_ListenerId lid,
 														   PL_StruxFmtHandle sfhNew));
+	fl_SectionLayout *  doclistener_insertFrame(const PX_ChangeRecord_Strux * pcrx,
+								   SectionType iType,
+								   PL_StruxDocHandle sdh,
+								   PL_ListenerId lid,
+								   void (* pfnBindHandles)(PL_StruxDocHandle sdhNew,
+														   PL_ListenerId lid,
+														   PL_StruxFmtHandle sfhNew));
 
 	bool doclistener_insertObject(const PX_ChangeRecord_Object * pcro);
 	bool doclistener_deleteObject(const PX_ChangeRecord_Object * pcro);
@@ -342,9 +310,21 @@ public:
 	virtual bool 		    needsRedraw(void) const
 		{ return m_bNeedsRedraw; }
 	virtual void			markAllRunsDirty(void);
+	UT_sint32               findLineInBlock(fp_Line * pLine);
 	bool					checkWord(fl_PartOfBlock* pPOB);
 	void					recheckIgnoredWords();
-
+	void                    setStyleInTOC(bool b)
+	{	m_bStyleInTOC = b;}
+	void                    forceSectionBreak(void);
+	bool                    isContainedByTOC(void) const
+	    { return m_bIsTOC;}
+	FootnoteType            getTOCNumType(void);
+	eTabLeader              getTOCTabLeader(UT_sint32 iOff);
+	UT_sint32               getTOCTabPosition(UT_sint32 iOff);
+	void                    setAccumHeight(UT_sint32 i)
+	{ m_iAccumulatedHeight =i;}
+	UT_sint32               getAccumHeight(void) const
+	{ return m_iAccumulatedHeight;}
 	static bool 		s_EnumTabStops(void * myThis, UT_uint32 k, fl_TabStop *pTabInfo);
 
 	inline void 		addBackgroundCheckReason(UT_uint32 reason) {m_uBackgroundCheckReasons |= reason;}
@@ -354,7 +334,14 @@ public:
 	// The following is a set of bit flags giving the reason this block is
 	// queued for background checking.	See specific values in fl_DocLayout.h
 	UT_uint32				m_uBackgroundCheckReasons;
-
+	void                    setPrevListLabel(bool b)
+	{ m_bPrevListLabel = b;}
+	bool                    getNextTableElement(UT_GrowBuf * buf,
+												PT_DocPosition startPos, 
+												PT_DocPosition & begPos,
+												PT_DocPosition & endPos,
+												UT_UTF8String & sWord,
+												bool bIgnoreSpace);
 #ifdef FMT_TEST
 	void					__dump(FILE * fp) const;
 #endif
@@ -368,6 +355,7 @@ protected:
 										 bool bClearScreen = true);
 
 	bool					_spellCheckWord(const UT_UCSChar * word, UT_uint32 len, UT_uint32 blockPos);
+	SpellChecker * _getSpellChecker (UT_uint32 blockPos);
 
 	bool					_truncateLayout(fp_Run* pTruncRun);
 
@@ -390,13 +378,18 @@ protected:
 	bool					_doInsertForcedColumnBreakRun(PT_BlockOffset blockOffset);
 	bool					_doInsertForcedPageBreakRun(PT_BlockOffset blockOffset);
 	bool					_doInsertTabRun(PT_BlockOffset blockOffset);
+	bool					_doInsertTOCTabRun(PT_BlockOffset blockOffset);
+	bool					_doInsertTOCListLabelRun(PT_BlockOffset blockOffset);
+	bool					_doInsertTOCHeadingRun(PT_BlockOffset blockOffset);
+	bool                    _doInsertTOCListTabRun(PT_BlockOffset blockOffset);
 	bool					_doInsertImageRun(PT_BlockOffset blockOffset, FG_Graphic* pFG);
 	bool					_doInsertFieldRun(PT_BlockOffset blockOffset, const PX_ChangeRecord_Object * pcro);
+	bool					_doInsertFieldTOCRun(PT_BlockOffset blockOffset);
 	bool                    _doInsertDirectionMarkerRun(PT_BlockOffset blockOffset, UT_UCS4Char iM);
 	bool					_deleteFmtMark(PT_BlockOffset blockOffset);
 
-	void					_lookupProperties(void);
-	void					_removeLine(fp_Line*);
+	virtual void			_lookupProperties(const PP_AttrProp* pAP);
+	void					_removeLine(fp_Line*, bool bRemoveFromContainer=false);
 	void                    _purgeLine(fp_Line*);
 	void					_removeAllEmptyLines(void);
 
@@ -416,14 +409,11 @@ protected:
 	void					_deleteListLabel(void);
 	inline void 			_addBlockToPrevList( fl_BlockLayout * prevBlockInList, UT_uint32 level);
 	inline void 			_prependBlockToPrevList( fl_BlockLayout * nextBlockInList);
-
 	UT_sint32 				m_iNeedsReformat; // will store offset
 											  // from which reformat
 											  // is need, -1 if not
 	bool					m_bNeedsRedraw;
-	bool					m_bFixCharWidths;
-	bool								 m_bIsHdrFtr;
-	fl_CharWidths			m_gbCharWidths;
+	bool				    m_bIsHdrFtr;
 
 	FL_DocLayout*			m_pLayout;
 	fb_LineBreaker 		    m_Breaker;
@@ -431,7 +421,7 @@ protected:
 	fp_Run* 				m_pFirstRun;
 	fl_SectionLayout*		m_pSectionLayout;
 
-	UT_Vector				m_vecTabs;
+	UT_GenericVector<fl_TabStop*>	m_vecTabs;
 	UT_sint32				m_iDefaultTabInterval;
 	// read-only caches of the underlying properties
 	UT_uint32				m_iOrphansProperty;
@@ -457,8 +447,19 @@ protected:
 	bool                    m_bIsCollapsed;
 	bool                    m_bHasUpdatableField;
 
-	FriBidiCharType 		m_iDomDirection;
-	FriBidiCharType 		m_iDirOverride;
+	UT_BidiCharType 		m_iDomDirection;
+	UT_BidiCharType 		m_iDirOverride;
+
+	bool                    m_bIsTOC;
+	bool                    m_bStyleInTOC;
+	UT_sint32               m_iTOCLevel;
+
+	bool                    m_bSameYAsPrevious;
+	UT_sint32               m_iAccumulatedHeight;
+	fp_VerticalContainer *  m_pVertContainer;
+	UT_sint32               m_iLinePosInContainer;
+	bool                    m_bForceSectionBreak;
+	bool                    m_bPrevListLabel;
 };
 
 /*
