@@ -2,8 +2,8 @@
 
 /* AbiWord
  * Copyright (C) 1998-2000 AbiSource, Inc.
- * Copyright (C) 2001 Hubert Figuiere
- * Copyright (C) 2002 Francis James Franklin <fjf@alinameridon.com>
+ * Copyright (C) 2001-2004 Hubert Figuiere
+ * Copyright (C) 2002-2004 Francis James Franklin
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,55 +31,62 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <dirent.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <signal.h>
+
 #include <popt.h>
 
+#include "ut_bytebuf.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
+#include "ut_string_class.h"
+#include "ut_math.h"
 #include "ut_misc.h"
+#include "ut_png.h"
+
 #include "ut_PerlBindings.h"
 #include "ut_Script.h"
 
-#include "xap_Args.h"
-#include "ap_Args.h"
-#include "ap_Convert.h"
-#include "ap_CocoaFrame.h"
-#include "ap_CocoaApp.h"
-#include "spell_manager.h"
-#include "ap_Strings.h"
-#include "xap_EditMethods.h"
-#include "ap_LoadBindings.h"
-#include "ap_CocoaClipboard.h"
-#include "ap_CocoaPrefs.h"
-#include "ap_CocoaSplash.h"
 #include "ev_CocoaMenuBar.h"
+#include "ev_EditMethod.h"
+
+#include "xap_Args.h"
+#include "xap_CocoaAppController.h"
 #include "xap_DialogFactory.h"
-#include "xap_Dlg_MessageBox.h"
 #include "xap_Dialog_Id.h"
+#include "xap_Dlg_MessageBox.h"
+#include "xap_EditMethods.h"
+#include "xap_EncodingManager.h"
 #include "xap_Menu_ActionSet.h"
 #include "xap_Menu_Layouts.h"
+#include "xap_Module.h"
+#include "xap_ModuleManager.h"
+#include "xap_Prefs.h"
 #include "xap_Toolbar_ActionSet.h"
 #include "xap_Toolbar_Layouts.h"
+
 #include "xav_View.h"
 
-#include "ev_EditMethod.h"
-#include "ev_CocoaMenuBar.h"
-#include "gr_Graphics.h"
 #include "gr_CocoaGraphics.h"
 #include "gr_CocoaImage.h"
+#include "gr_Graphics.h"
 #include "gr_Image.h"
-#include "ut_bytebuf.h"
-#include "ut_png.h"
-#include "ut_debugmsg.h"
 
-#include "fv_View.h"
 #include "fp_Run.h"
 
-#include "ut_string_class.h"
-#include "xap_EncodingManager.h"
+#include "fv_View.h"
+
+#include "ap_Args.h"
+#include "ap_CocoaApp.h"
+#include "ap_CocoaClipboard.h"
+#include "ap_CocoaFrame.h"
+#include "ap_CocoaPrefs.h"
+#include "ap_CocoaSplash.h"
+#include "ap_Convert.h"
+#include "ap_LoadBindings.h"
+#include "ap_Prefs_SchemeIds.h"
+#include "ap_Strings.h"
 
 #include "ie_impexp_Register.h"
 
@@ -90,15 +97,10 @@
 #include "ie_imp.h"
 #include "ie_imp_RTF.h"
 #include "ie_imp_Text.h"
+#include "ie_impGraphic.h"
 
-#include "xap_Prefs.h"
-#include "ap_Prefs_SchemeIds.h"
-#include "gr_Image.h"
+#include "spell_manager.h"
 
-#include "xap_Module.h"
-#include "xap_ModuleManager.h"
-
-#import "xap_CocoaAppController.h"
 
 
 // quick hack - this is defined in ap_EditMethods.cpp
@@ -179,26 +181,69 @@ static bool s_createDirectoryIfNecessary(const char * szDir, bool publicdir = fa
 */
 bool AP_CocoaApp::initialize(void)
 {
+	static const char * suffix = "/Library/Application Support/AbiSuite";
+
+	if (s_createDirectoryIfNecessary(suffix, true)) // let's create some system-level directories also, if we can
+		{
+			UT_UTF8String path(suffix);
+
+			path += "/dictionary";
+			s_createDirectoryIfNecessary(path.utf8_str(), true);
+
+			path  = suffix;
+			path += "/templates";
+			s_createDirectoryIfNecessary(path.utf8_str(), true);
+
+			path  = suffix;
+			path += "/Plug-ins";
+			s_createDirectoryIfNecessary(path.utf8_str(), true);
+		}
+
     const char * szUserPrivateDirectory = getUserPrivateDirectory();
 
     bool bVerified = false;
+
 	if (szUserPrivateDirectory)
 		{
-			static const char * suffix = "/Application Support/AbiSuite";
-			int suffix_length = strlen (suffix);
-			int usrprv_length = strlen (szUserPrivateDirectory);
-			if (usrprv_length > suffix_length)
-				if (strcmp (szUserPrivateDirectory + (usrprv_length - suffix_length), suffix) == 0)
-					{
-						UT_String path(szUserPrivateDirectory, usrprv_length - suffix_length);
+			int suffix_length = strlen(suffix);
+			int usrprv_length = strlen(szUserPrivateDirectory);
 
-						if (s_createDirectoryIfNecessary (path.c_str ()))
+			if (usrprv_length > suffix_length)
+				if (strcmp(szUserPrivateDirectory + (usrprv_length - suffix_length), suffix) == 0)
+					{
+						UT_UTF8String path(szUserPrivateDirectory, usrprv_length - suffix_length);
+
+						if (s_createDirectoryIfNecessary(path.utf8_str()))
 							{
-								path += "/Application Support";
-								if (s_createDirectoryIfNecessary (path.c_str ()))
+								path += "/Library";
+								if (s_createDirectoryIfNecessary(path.utf8_str()))
 									{
-										path += "/AbiSuite";
-										if (s_createDirectoryIfNecessary (path.c_str ())) bVerified = true;
+										path += "/Application Support";
+										if (s_createDirectoryIfNecessary(path.utf8_str()))
+											{
+												UT_UTF8String path2(path);
+
+												path2 += "/Enchant";
+												s_createDirectoryIfNecessary(path2.utf8_str());
+
+												path += "/AbiSuite";
+												if (s_createDirectoryIfNecessary(path.utf8_str()))
+													{
+														bVerified = true;
+
+														path2  = path;
+														path2 += "/dictionary";
+														s_createDirectoryIfNecessary(path2.utf8_str());
+
+														path2  = path;
+														path2 += "/templates";
+														s_createDirectoryIfNecessary(path2.utf8_str());
+
+														path2  = path;
+														path2 += "/Plug-ins";
+														s_createDirectoryIfNecessary(path2.utf8_str());
+													}
+											}
 									}
 							}
 					}
@@ -228,7 +273,7 @@ bool AP_CocoaApp::initialize(void)
     m_pToolbarActionSet = AP_CreateToolbarActionSet();
     UT_ASSERT(m_pToolbarActionSet);
     
-    if (! XAP_CocoaApp::initialize())
+    if (! AP_App::initialize())
 		return false;
 
 	//////////////////////////////////////////////////////////////////
@@ -250,7 +295,7 @@ bool AP_CocoaApp::initialize(void)
 	    
 		// see if we should load an alternative set from the disk
 	    
-		const char * szDirectory = NULL;
+//		const char * szDirectory = NULL;
 		const char * szStringSet = NULL;
 	    
 		if (   (getPrefsValue(AP_PREF_KEY_StringSet,
@@ -259,6 +304,7 @@ bool AP_CocoaApp::initialize(void)
 			   && (*szStringSet)
 			   && (strcmp(szStringSet,AP_PREF_DEFAULT_StringSet) != 0))
 		{
+#if 0			
 			getPrefsValueDirectory(true,
 								   (const XML_Char*)AP_PREF_KEY_StringSetDirectory,
 								   (const XML_Char**)&szDirectory);
@@ -269,20 +315,24 @@ bool AP_CocoaApp::initialize(void)
 				szPathname += "/";
 			szPathname += szStringSet;
 			szPathname += ".strings";
-		
+#endif
+
+			NSString* resources = [[NSBundle mainBundle] resourcePath];
+			NSString* stringSet = [resources stringByAppendingPathComponent:[NSString stringWithFormat:@"AbiWord/strings/%s%@",szStringSet,@".strings"]];
+
 			AP_DiskStringSet * pDiskStringSet = new AP_DiskStringSet(this);
 			UT_ASSERT(pDiskStringSet);
 		
-			if (pDiskStringSet->loadStringsFromDisk(szPathname.c_str()))
+			if (pDiskStringSet->loadStringsFromDisk([stringSet UTF8String]))
 			{
 				pDiskStringSet->setFallbackStringSet(m_pStringSet);
 				m_pStringSet = pDiskStringSet;
-				UT_DEBUGMSG(("Using StringSet [%s]\n",szPathname.c_str()));
+				UT_DEBUGMSG(("Using StringSet [%s]\n",[stringSet UTF8String]));
 			}
 			else
 			{
 				DELETEP(pDiskStringSet);
-				UT_DEBUGMSG(("Unable to load StringSet [%s] -- using builtin strings instead.\n",szPathname.c_str()));
+				UT_DEBUGMSG(("Unable to load StringSet [%s] -- using builtin strings instead.\n",[stringSet UTF8String]));
 			}
 		}
     }
@@ -317,6 +367,9 @@ bool AP_CocoaApp::initialize(void)
 	}
 	FREEP(m_szMenuLabelSetName);
 	m_szMenuLabelSetName = UT_strdup(szMenuLabelSetName);
+	UT_ASSERT(m_szMenuLabelSetName);
+	if (!m_szMenuLabelSetName)
+		return false;
 	
 	getMenuFactory()->buildMenuLabelSet(m_szMenuLabelSetName);
 	const char * szMenuLayoutName = NULL;
@@ -329,21 +382,23 @@ bool AP_CocoaApp::initialize(void)
 	}
 	FREEP(m_szMenuLayoutName);
 	m_szMenuLayoutName = UT_strdup(szMenuLayoutName);
+	UT_ASSERT(m_szMenuLayoutName);
+	if (!m_szMenuLayoutName)
+		return false;
 
 	// synthesize a menu from the info in our base class.
 
-	m_pCocoaMenu = new EV_CocoaMenuBar(this, m_szMenuLayoutName,
-	                                                 m_szMenuLabelSetName);
+	m_pCocoaMenu = new EV_CocoaMenuBar(this, m_szMenuLayoutName, m_szMenuLabelSetName);
 	UT_ASSERT(m_pCocoaMenu);
-	bool bResult = m_pCocoaMenu->synthesizeMenuBar([XAP_AppController_Instance getMenuBar]);
-	UT_ASSERT(bResult);
-	
-	
+	if (!m_pCocoaMenu)
+		return false;
+
 	bool bLoadPlugins = true;
 	bool bFound = getPrefsValueBool(XAP_PREF_KEY_AutoLoadPlugins,&bLoadPlugins);
 	if(bLoadPlugins || !bFound)
 	{
-		loadAllPlugins();
+		XAP_CocoaAppController * pController = (XAP_CocoaAppController *) [NSApp delegate];
+		[pController setAutoLoadPluginsAfterLaunch:YES];
 	}
     //////////////////////////////////////////////////////////////////
 
@@ -438,6 +493,9 @@ bool AP_CocoaApp::getPrefsValueDirectory(bool bAppSpecific,
 */
 const char * AP_CocoaApp::getAbiSuiteAppDir(void) const
 {
+	static const char * SystemAppDir = "/Library/Application Support/AbiSuite/AbiWord"; // FIXME ??
+	return SystemAppDir;
+#if 0
     // we return a static string, use it quickly.
 	
     static XML_Char buf[1024];
@@ -445,6 +503,7 @@ const char * AP_CocoaApp::getAbiSuiteAppDir(void) const
 
     sprintf(buf,"%s/%s",getAbiSuiteLibDir(),ABIWORD_APP_LIBDIR);
     return buf;
+#endif
 }
 
 /*!
@@ -487,12 +546,12 @@ void AP_CocoaApp::copyToClipboard(PD_DocumentRange * pDocRange, bool bUseClipboa
 
     // create raw 8bit text buffer to put on the clipboard
 		
-    IE_Exp_Text * pExpText = new IE_Exp_Text(pDocRange->m_pDoc);
+    IE_Exp_Text * pExpText = new IE_Exp_Text(pDocRange->m_pDoc, "UTF-8");
     if (pExpText)
     {
 		pExpText->copyToBuffer(pDocRange,&bufTEXT);
 		DELETEP(pExpText);
-		UT_DEBUGMSG(("CopyToClipboard: copying %d bytes in TEXTPLAIN format.\n",bufTEXT.getLength()));
+		UT_DEBUGMSG(("CopyToClipboard: copying %d bytes in TEXTPLAIN (UTF-8) format.\n",bufTEXT.getLength()));
     }
 
     // NOTE: this clearData() will actually release our ownership of
@@ -514,39 +573,20 @@ void AP_CocoaApp::copyToClipboard(PD_DocumentRange * pDocRange, bool bUseClipboa
     return;
 }
 
-/*
-  I've reordered AP_CLIPBOARD_STRING and AP_CLIPBOARD_TEXTPLAIN_8BIT
-  since for non-Latin1 text the data in AP_CLIPBOARD_TEXTPLAIN_8BIT
-  format has name of encoding as prefix, and AP_CLIPBOARD_STRING
-  doesn't - hvv.
-*/
 static const char * aszFormatsAccepted[] = { XAP_CocoaClipboard::XAP_CLIPBOARD_RTF,
 											 XAP_CocoaClipboard::XAP_CLIPBOARD_STRING,
 											 XAP_CocoaClipboard::XAP_CLIPBOARD_TEXTPLAIN_8BIT,
+											 XAP_CocoaClipboard::XAP_CLIPBOARD_IMAGE,
 											 0 /* must be last */ };
-static const char * txtszFormatsAccepted[] = { XAP_CocoaClipboard::XAP_CLIPBOARD_STRING,
-											   XAP_CocoaClipboard::XAP_CLIPBOARD_TEXTPLAIN_8BIT,
-											   0 };
 
 /*!
   paste from the system clipboard using the best-for-us format
   that is present.  try to get the content in the order listed.
-  
-  \todo currently i have this set so that a ^v or Menu[Edit/Paste] will
-  use the CLIPBOARD property and a MiddleMouseClick will use the
-  PRIMARY property -- this seems to be the "X11 way" (sigh).
-  consider having a preferences switch to allow ^v and Menu[Edit/Paste]
-  to use the most recent property... this might be a nice way of
-  unifying things -- or it might not -- this is probably an area
-  for investigation or some usability testing.
 */
+// FIXME: this code is butt ugly.
 void AP_CocoaApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipboard,
 									bool bHonorFormatting)
 {
-//    XAP_CocoaClipboard::T_AllowGet tFrom = ((bUseClipboard)
-//										   ? XAP_CocoaClipboard::TAG_ClipboardOnly
-//										   : XAP_CocoaClipboard::TAG_PrimaryOnly);
-
     const char * szFormatFound = NULL;
     unsigned char * pData = NULL;
     UT_uint32 iLen = 0;
@@ -567,18 +607,8 @@ void AP_CocoaApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClip
 		pImpRTF->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpRTF);
 
-		return;
     }
-
-	bFoundOne = m_pClipboard->getClipboardData(/*tFrom,*/txtszFormatsAccepted,(void**)&pData,&iLen,&szFormatFound);	
-
-	if (!bFoundOne)
-	{
-		UT_DEBUGMSG(("PasteFromClipboard: did not find anything to paste.\n"));
-		return;
-	}
-
-    if (   (strcmp(szFormatFound, XAP_CocoaClipboard::XAP_CLIPBOARD_TEXTPLAIN_8BIT) == 0)
+	else if (   (strcmp(szFormatFound, XAP_CocoaClipboard::XAP_CLIPBOARD_TEXTPLAIN_8BIT) == 0)
 		   || (strcmp(szFormatFound, XAP_CocoaClipboard::XAP_CLIPBOARD_STRING) == 0))
     {
 		iLen = UT_MIN(iLen,strlen((const char *)pData));
@@ -588,9 +618,56 @@ void AP_CocoaApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClip
 		pImpText->pasteFromBuffer(pDocRange,pData,iLen);
 		DELETEP(pImpText);
 
-		return;
     }
+	else if (strcmp(szFormatFound, XAP_CocoaClipboard::XAP_CLIPBOARD_IMAGE) == 0) {
+		  IE_ImpGraphic * pIEG = NULL;
+		  FG_Graphic * pFG = NULL;
+		  IEGraphicFileType iegft = IEGFT_Unknown;
+		  UT_Error error = UT_OK;
+		  
+		  XAP_Frame * pFrame = getLastFocussedFrame ();
+		  
+		  UT_ByteBuf * bytes = new UT_ByteBuf( iLen );
+		  
+		  bytes->append (pData, iLen);
+		  
+		  error = IE_ImpGraphic::constructImporter(bytes, iegft, &pIEG);
+		  if(error)
+		  {
+			  UT_DEBUGMSG(("DOM: could not construct importer (%d)\n", 
+						   error));
+			  DELETEP(bytes);
+			  return;
+		  }
+		  
+		  error = pIEG->importGraphic(bytes, &pFG);
+		  if(!pFG || error)
+		  {
+			  UT_DEBUGMSG(("DOM: could not import graphic (%d)\n", error));
+			  DELETEP(bytes);
+			  DELETEP(pIEG);
+			  return;
+		  }
+		  
+		  // at this point, 'bytes' is owned by pFG
+		  FV_View * pView = static_cast<FV_View*>(pFrame->getCurrentView());
+		  
+		  UT_UTF8String newName = UT_UTF8String_sprintf ( "paste_image_%d", UT_newNumber() ) ;
+		  
+		  DELETEP(pIEG);
+		  
+		  error = pView->cmdInsertGraphic(pFG);
+		  if (error)
+		  {
+			  UT_DEBUGMSG(("DOM: could not insert graphic (%d)\n", error));
+			  DELETEP(pFG);
+			  return;
+		  }
+		  
+		  DELETEP(pFG);	
+	}
 
+	FREEP(pData);
     return;
 }
 
@@ -605,164 +682,15 @@ void AP_CocoaApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClip
 */
 bool AP_CocoaApp::canPasteFromClipboard(void)
 {
-#if 0
     const char * szFormatFound = NULL;
     unsigned char * pData = NULL;
     UT_uint32 iLen = 0;
 
-    XAP_CocoaClipboard::T_AllowGet tFrom = XAP_CocoaClipboard::TAG_ClipboardOnly;
-
     // first, try to see if we can paste from the clipboard
-    bool bFoundOne = m_pClipboard->getData(tFrom,aszFormatsAccepted,(void**)&pData,&iLen,&szFormatFound);
-    if (bFoundOne)
-		return true;
-
-    // didn't work, try out the primary selection
-    tFrom = XAP_CocoaClipboard::TAG_PrimaryOnly;
-    bFoundOne = m_pClipboard->getData(tFrom,aszFormatsAccepted,(void**)&pData,&iLen,&szFormatFound);
-    return bFoundOne;
-#else
-    return true;
-#endif
-}
-
-/* return > 0 for directory entries ending in ".Abi"
- */
-static int s_Abi_only (struct dirent * d)
-{
-  const char * name = d->d_name;
-
-  if (name)
-    {
-      int length = strlen (name);
-
-      if (length >= 4)
-	if (strcmp (name + length - 4, ".Abi") == 0)
-	  return 1;
-    }
-  return 0;
-}
-
-/* return true if dirname exists and is a directory; symlinks probably not counted
- */
-static bool s_dir_exists (const char * dirname)
-{
-  struct stat sbuf;
-
-  if (stat (dirname, &sbuf) == 0)
-    if ((sbuf.st_mode & S_IFMT) == S_IFDIR)
-      return true;
-
-  return false;
-}
-
-/* MacOSX applications look for plugins in Contents/Plug-ins, and there's probably
- * no need to jump through scandir hoops identifying these. Third party plugins or
- * plugins not distributed with AbiWord.app can be found in the system directory
- * "/Library/Application Support" or in the user's home equivalent - I'm choosing
- * to make the plug-in directory "/Library/Application Support/AbiWord/Plug-ins".
- * 
- * The System Overview recommends that plugins be given a suffix that the
- * application claims (presumably as a document type) so I'm opting for the suffix
- * ".Abi".
- */
-void AP_CocoaApp::loadAllPlugins ()
-{
-	/* 1. TODO: Load from AbiWord.app/Contents/Plug-ins
-	 */
-	NSString * app_path = [[NSBundle mainBundle] bundlePath];
-	if (app_path)
-		{
-			NSString * plugin_path = [app_path stringByAppendingString:@"/Contents/Plug-ins"];
-			UT_DEBUGMSG(("FJF: path to bundle's plug-ins: %s\n",[plugin_path lossyCString]));
-		}
-
-	/* 2. Load from:
-	 *  a. "/Library/Application Support/AbiSuite/Plug-ins"
-	 *  b. "$HOME/Library/Application Support/AbiSuite/Plug-ins"
-	 */
-	int support_dir_count = 0;
-
-	UT_String support_dir[2];
-
-	support_dir[0] = "/Library/Application Support/AbiSuite/Plug-ins";
-
-	/* create the system plugins directory - if we can...
-	 */
-	if (s_createDirectoryIfNecessary ("/Library", true))
-		if (s_createDirectoryIfNecessary ("/Library/Application Support", true))
-			if (s_createDirectoryIfNecessary ("/Library/Application Support/AbiSuite", true))
-				s_createDirectoryIfNecessary ("/Library/Application Support/AbiSuite/Plug-ins", true);
-
-	if (!s_dir_exists (support_dir[0].c_str()))
-		{
-			UT_DEBUGMSG(("FJF: %s: no such directory\n",support_dir[0].c_str()));
-		}
-	else
-		{
-			UT_DEBUGMSG(("FJF: adding to plug-in search path: %s\n",support_dir[0].c_str()));
-			support_dir_count++;
-		}
-
-	const char * homedir = getUserPrivateDirectory ();
-	if (homedir == 0)
-		{
-			UT_DEBUGMSG(("FJF: no home directory?\n"));
-		}
-	else if (!s_dir_exists (homedir))
-		{
-			UT_DEBUGMSG(("FJF: invalid home directory?\n"));
-		}
-	else
-		{
-			UT_String plugin_dir(homedir);
-			plugin_dir += "/Plug-ins";
-			if (!s_createDirectoryIfNecessary (plugin_dir.c_str()))
-				{
-					UT_DEBUGMSG(("FJF: %s: no such directory\n",plugin_dir.c_str()));
-				}
-			else
-				{
-					UT_DEBUGMSG(("FJF: adding to plug-in search path: %s\n",plugin_dir.c_str()));
-					support_dir[support_dir_count++] = plugin_dir;
-				}
-		}
-
-	for (int i = 0; i < support_dir_count; i++)
-		{
-			struct dirent ** namelist = 0;
-			int n = scandir (support_dir[i].c_str(), &namelist, s_Abi_only, alphasort);
-			UT_DEBUGMSG(("DOM: found %d plug-ins in %s\n", n, support_dir[i].c_str()));
-			if (n == 0) {
-				FREEP (namelist);
-				continue;
-			}
-			UT_String plugin_path;
-			while (n--)
-				{
-					plugin_path  = support_dir[i];
-					plugin_path += '/';
-					plugin_path += namelist[n]->d_name;
-
-					UT_DEBUGMSG(("DOM: loading plugin %s\n", plugin_path.c_str()));
-					if (XAP_ModuleManager::instance().loadModule (plugin_path.c_str()))
-						{
-							UT_DEBUGMSG(("DOM: loaded plug-in: %s\n", namelist[n]->d_name));
-						}
-					else
-						{
-							UT_DEBUGMSG(("DOM: didn't load plug-in: %s\n", namelist[n]->d_name));
-						}
-
-					free (namelist[n]);
-				}
-			free (namelist);
-		}
-
-	/* SPI modules don't register automatically on loading, so
-	 * now that we've loaded the modules we need to register them:
-	 */
-	XAP_ModuleManager::instance().registerPending ();
+    bool bFoundOne = m_pClipboard->getClipboardData(aszFormatsAccepted,(void**)&pData,&iLen,&szFormatFound);
+	FREEP(pData);
+	
+	return bFoundOne;
 }
 
 /*****************************************************************/
@@ -1041,8 +969,12 @@ int AP_CocoaApp::main(const char * szAppName, int argc, const char ** argv)
     UT_DEBUGMSG(("Compile Date:\t%s\n", XAP_App::s_szBuild_CompileDate));
     UT_DEBUGMSG(("Compile Time:\t%s\n", XAP_App::s_szBuild_CompileTime));
     
+	/* we create this autorelease pool because app startup will create a lot of things */
+	/* we'll delete it before starting the main loop to cleanup memory */
+	NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
+
     // initialize our application.
-	[NSApplication sharedApplication];
+	[XAP_CocoaApplication sharedApplication];
 	
     XAP_Args XArgs = XAP_Args(argc,argv);
 	AP_CocoaApp * pMyCocoaApp = new AP_CocoaApp(&XArgs, szAppName);
@@ -1104,32 +1036,35 @@ int AP_CocoaApp::main(const char * szAppName, int argc, const char ** argv)
     // continue out the door.
 	// We used to check for bShowApp here.  It shouldn't be needed
 	// anymore, because doWindowlessArgs was supposed to bail already. -PL
-    if (pMyCocoaApp->openCmdLineFiles(&Args))
+    // if (pMyCocoaApp->openCmdLineFiles(&Args))
+
+	if (true) // really don't want to be opening files atm anyway
     {
+		[pool release];
+
 		// turn over control to Cocoa
 		[NSApp run];
+
+		pool = [[NSAutoreleasePool alloc] init];
 	}
     else
 	{
 		UT_DEBUGMSG(("DOM: not parsing command line or showing app\n"));
 	}
     
-    // destroy the App.  It should take care of deleting all frames.
+    /* destroy the App.  It should take care of deleting all frames.
+	 */
     pMyCocoaApp->shutdown();
+
     delete pMyCocoaApp;
     
+	if (pool)
+	{
+		[pool release];
+	}
     return 0;
 }
 
-XAP_Frame * AP_CocoaApp::newFrame(AP_App * app)
-{
-	AP_CocoaFrame * pFrame = new AP_CocoaFrame(app);
-	if (pFrame)
-		pFrame->initialize();
-
-	return pFrame;
-}
-
 void AP_CocoaApp::errorMsgBadArg(AP_Args * Args, int nextopt)
 {
 	printf ("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n",
@@ -1163,7 +1098,7 @@ bool AP_CocoaApp::doWindowlessArgs(const AP_Args *Args)
 		bool bFound = false;
 		if(Args->m_sPlugin != NULL)
 		{
-			const UT_Vector * pVec = XAP_ModuleManager::instance().enumModules ();
+			const UT_GenericVector<XAP_Module*>* pVec = XAP_ModuleManager::instance().enumModules ();
 			for (UT_uint32 i = 0; (i < pVec->size()) && !bFound; i++)
 			{
 				pModule = (XAP_Module *)pVec->getNthItem (i);
@@ -1241,11 +1176,12 @@ void AP_CocoaApp::catchSignals(int sig_num)
     UT_DEBUGMSG(("Oh no - we just segfaulted!\n"));
 	
     UT_uint32 i = 0;
+	IEFileType abiType = IE_Imp::fileTypeForSuffix("abw");
     for(;i<m_vecFrames.getItemCount();i++)
     {
 		AP_CocoaFrame * curFrame = (AP_CocoaFrame*) m_vecFrames[i];
 		UT_ASSERT(curFrame);
-		curFrame->backup(".CRASHED");
+		curFrame->backup(".CRASHED", abiType);
     }
     
     fflush(stdout);
