@@ -18,8 +18,10 @@
  */
 
 #undef GTK_DISABLE_DEPRECATED
+
 #include <stdlib.h>
 #include <glade/glade.h>
+#include "ut_locale.h"
 
 #include "ut_string.h"
 #include "ut_assert.h"
@@ -98,6 +100,14 @@ static void s_border_color(GtkWidget *widget, gpointer data )
 	dlg->event_previewExposed();
 }
 
+
+static void s_border_thickness(GtkWidget *widget, gpointer data )
+{
+	AP_UnixDialog_FormatTable * dlg = static_cast<AP_UnixDialog_FormatTable *>(data);
+	UT_return_if_fail(widget && dlg);
+	dlg->event_BorderThicknessChanged();
+}
+
 static void s_background_color(GtkWidget *widget, gpointer data)
 {
 	AP_UnixDialog_FormatTable * dlg = static_cast<AP_UnixDialog_FormatTable *>(data);
@@ -105,7 +115,7 @@ static void s_background_color(GtkWidget *widget, gpointer data)
 	
 	guint8 r, g, b, a;
 	gtk_color_picker_get_i8 (GTK_COLOR_PICKER(widget), &r, &g, &b, &a);
-	dlg->setBGColor(UT_RGBColor(r,g,b));
+	dlg->setBackgroundColor(UT_RGBColor(r,g,b));
 	dlg->event_previewExposed();
 }
 
@@ -124,6 +134,23 @@ static gboolean s_apply_to_changed(GtkWidget *widget, gpointer data)
 	return FALSE;
 }
 
+
+static gboolean s_select_image(GtkWidget *widget, gpointer data)
+{
+	AP_UnixDialog_FormatTable * dlg = reinterpret_cast<AP_UnixDialog_FormatTable *>(data);
+	UT_return_val_if_fail(widget && dlg, FALSE);
+	dlg->askForGraphicPathName();
+	return FALSE;
+}
+
+static gboolean s_remove_image(GtkWidget *widget, gpointer data)
+{
+	AP_UnixDialog_FormatTable * dlg = reinterpret_cast<AP_UnixDialog_FormatTable *>(data);
+	UT_return_val_if_fail(widget && dlg, FALSE);
+	dlg->clearImage();
+	return FALSE;
+}
+
 /*****************************************************************/
 
 #define	WIDGET_ID_TAG_KEY "id"
@@ -151,6 +178,22 @@ AP_UnixDialog_FormatTable::AP_UnixDialog_FormatTable(XAP_DialogFactory * pDlgFac
 	m_wLineTop = NULL;
 	m_wLineBottom = NULL;
 	m_wApplyToMenu = NULL;	
+	m_wSelectImageButton = NULL;
+	m_wNoImageButton = NULL;
+	m_wBorderThickness = NULL;
+	m_iBorderThicknessConnect = 0;
+//
+// These are hardwired into the GUI.
+//
+	const char * sThickness[FORMAT_TABLE_NUMTHICKNESS] ={"0.25pt","0.5pt",
+													   "0.75pt","1.0pt",
+													   "1.5pt","2.25pt","3pt",
+													   "4.5pt","6.0pt"};
+	UT_sint32 i = 0;
+	for(i=0; i< FORMAT_TABLE_NUMTHICKNESS ;i++)
+	{
+		m_dThickness[i] = UT_convertToInches(sThickness[i]);
+	}
 }
 
 AP_UnixDialog_FormatTable::~AP_UnixDialog_FormatTable(void)
@@ -176,7 +219,9 @@ void AP_UnixDialog_FormatTable::runModeless(XAP_Frame * pFrame)
 
 	// make a new Unix GC
 	DELETEP (m_pPreviewWidget);
-	m_pPreviewWidget = new GR_UnixGraphics(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	//m_pPreviewWidget = new GR_UnixGraphics(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	GR_UnixAllocInfo ai(m_wPreviewArea->window, unixapp->getFontManager(), m_pApp);
+	m_pPreviewWidget = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
 
 	// Todo: we need a good widget to query with a probable
 	// Todo: non-white (i.e. gray, or a similar bgcolor as our parent widget)
@@ -197,11 +242,15 @@ void AP_UnixDialog_FormatTable::runModeless(XAP_Frame * pFrame)
 void AP_UnixDialog_FormatTable::setSensitivity(bool bSens)
 {
 	gtk_widget_set_sensitive(m_wBorderColorButton, bSens);
+	gtk_widget_set_sensitive(m_wBorderThickness, bSens);
 	gtk_widget_set_sensitive(m_wBackgroundColorButton, bSens);	
+	gtk_widget_set_sensitive(m_wSelectImageButton, bSens);
+	gtk_widget_set_sensitive(m_wNoImageButton, bSens);
 	gtk_widget_set_sensitive(m_wLineLeft, bSens);
 	gtk_widget_set_sensitive(m_wLineRight, bSens);
 	gtk_widget_set_sensitive(m_wLineTop, bSens);
 	gtk_widget_set_sensitive(m_wLineBottom, bSens);
+	gtk_widget_set_sensitive(m_wApplyToMenu, bSens);
 	gtk_widget_set_sensitive(m_wApplyButton, bSens);
 }
 
@@ -217,6 +266,52 @@ void AP_UnixDialog_FormatTable::event_previewExposed(void)
 		m_pFormatTablePreview->draw();
 }
 
+void AP_UnixDialog_FormatTable::setBorderThicknessInGUI(UT_UTF8String & sThick)
+{
+	double thickness = UT_convertToInches(sThick.utf8_str());
+	guint i =0;
+	guint closest = 0;
+	double dClose = 100000000.;
+	for(i=0; i<FORMAT_TABLE_NUMTHICKNESS; i++)
+	{
+		double diff = thickness - m_dThickness[i];
+		if(diff < 0)
+			diff = -diff;
+		if(diff < dClose)
+		{
+			closest = i;
+			dClose = diff;
+		}
+	}
+	g_signal_handler_block(G_OBJECT(m_wBorderThickness),m_iBorderThicknessConnect);
+	gtk_option_menu_set_history (GTK_OPTION_MENU(m_wBorderThickness),
+							  closest);
+	g_signal_handler_unblock(G_OBJECT(m_wBorderThickness),m_iBorderThicknessConnect);
+}
+
+void AP_UnixDialog_FormatTable::setBackgroundColorInGUI(UT_RGBColor clr)
+{
+	gtk_color_picker_set_i8(GTK_COLOR_PICKER(m_wBackgroundColorButton), clr.m_red, clr.m_grn, clr.m_blu, 0 /* unused */);
+}
+
+void AP_UnixDialog_FormatTable::event_BorderThicknessChanged(void)
+{
+	if(m_wBorderThickness)
+	{
+		gint history = gtk_option_menu_get_history(GTK_OPTION_MENU(m_wBorderThickness));
+		double thickness = m_dThickness[history];
+
+		UT_UTF8String sThickness;
+		{
+			UT_LocaleTransactor(LC_NUMERIC, "C");
+			sThickness = UT_UTF8String_sprintf("%fin",thickness);
+		}
+
+		setBorderThickness(sThickness);
+		event_previewExposed();
+	}
+}
+
 void AP_UnixDialog_FormatTable::event_ApplyToChanged(void)
 {
 	if (m_wApplyToMenu)
@@ -318,9 +413,23 @@ GtkWidget * AP_UnixDialog_FormatTable::_constructWindow(void)
 	
 	localizeLabelMarkup(glade_xml_get_widget(xml, "lbBorder"), pSS, AP_STRING_ID_DLG_FormatTable_Borders);
 	localizeLabel(glade_xml_get_widget(xml, "lbBorderColor"), pSS, AP_STRING_ID_DLG_FormatTable_Color);
+	localizeLabel(glade_xml_get_widget(xml, "lbBorderThickness"), pSS, AP_STRING_ID_DLG_FormatTable_Thickness);
 	
 	localizeLabelMarkup(glade_xml_get_widget(xml, "lbBackground"), pSS, AP_STRING_ID_DLG_FormatTable_Background);
 	localizeLabel(glade_xml_get_widget(xml, "lbBackgroundColor"), pSS, AP_STRING_ID_DLG_FormatTable_Color);
+
+	
+	localizeLabelMarkup(glade_xml_get_widget(xml, "lbSetImageBackground"), pSS, AP_STRING_ID_DLG_FormatTable_SetImageBackground);
+	
+
+//	add the buttons for background image to the dialog.
+
+	m_wSelectImageButton = glade_xml_get_widget(xml, "btSelectImage");
+	m_wNoImageButton = glade_xml_get_widget(xml, "btNoImageBackground");
+	
+	localizeLabel(glade_xml_get_widget(xml, "lbSelectImage"), pSS, AP_STRING_ID_DLG_FormatTable_SelectImage);
+	
+	localizeLabel(glade_xml_get_widget(xml, "lbSetNoImage"), pSS, AP_STRING_ID_DLG_FormatTable_NoImageBackground);
 	
 	localizeLabelMarkup(glade_xml_get_widget(xml, "lbPreview"), pSS, AP_STRING_ID_DLG_FormatTable_Preview);
 
@@ -329,15 +438,20 @@ GtkWidget * AP_UnixDialog_FormatTable::_constructWindow(void)
 	// add the custom color picker buttons to the dialog
 	m_wBorderColorButton = gtk_color_picker_new();
 	gtk_widget_show(m_wBorderColorButton);
-	gtk_table_attach(GTK_TABLE (glade_xml_get_widget(xml, "tbBorderContent")), m_wBorderColorButton, 1, 2, 0, 1,
+	gtk_table_attach(GTK_TABLE (glade_xml_get_widget(xml, "tbBorder")), m_wBorderColorButton, 3, 4, 1, 2,
                     (GtkAttachOptions) (GTK_FILL), (GtkAttachOptions) (0), 
 					0, 0);
 					
 	// add the custom color picker buttons to the dialog
 	m_wBackgroundColorButton = gtk_color_picker_new();
 	gtk_widget_show(m_wBackgroundColorButton);
-	gtk_box_pack_start(GTK_BOX (glade_xml_get_widget(xml, "hbBackgroundColor")), m_wBackgroundColorButton, FALSE, TRUE, 0);
-	
+	gtk_box_pack_start(GTK_BOX (glade_xml_get_widget(xml, "hbBackgroundColor")), m_wBackgroundColorButton, TRUE, TRUE, 0);	
+
+//
+// Now the Border Thickness Option menu
+// 
+	m_wBorderThickness = glade_xml_get_widget(xml, "omBorderThickness");
+
 	// add the options to the "Apply to" menu
 	// NOTE: if you change this order, make sure to adjust event_ApplyToChanged as well!
 	// FIXME: PLEASE ADD A "localizeMenuItem" HELPER FUNCTION OR SOMETHING LIKE THAT
@@ -345,27 +459,32 @@ GtkWidget * AP_UnixDialog_FormatTable::_constructWindow(void)
 	GtkWidget * menu = gtk_menu_new();
 	
 	XML_Char * unixstr = NULL;	// used for conversions
-	
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Selection).utf8_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Selection,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_menu_item_new_with_label(unixstr));
 	FREEP(unixstr);	
 
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Row).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Row,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_menu_item_new_with_label(unixstr));
 	FREEP(unixstr);	
 
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Column).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Column,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_menu_item_new_with_label(unixstr));
 	FREEP(unixstr);	
 
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Table).utf8_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_FormatTable_Apply_To_Table,s);
+	UT_XML_cloneNoAmpersands(unixstr, s.utf8_str());
 	gtk_menu_shell_append(GTK_MENU_SHELL(menu), gtk_menu_item_new_with_label(unixstr));
 	FREEP(unixstr);	
 
 	gtk_option_menu_set_menu(GTK_OPTION_MENU(m_wApplyToMenu), menu);
 	gtk_option_menu_set_history(GTK_OPTION_MENU(m_wApplyToMenu), 0);
 	gtk_widget_show_all(menu);
-	
+
+
 	// add the apply and ok buttons to the dialog
 	m_wCloseButton = glade_xml_get_widget(xml, "btClose");
 	m_wApplyButton = glade_xml_get_widget(xml, "btApply");
@@ -406,6 +525,16 @@ void AP_UnixDialog_FormatTable::_connectSignals(void)
 							G_CALLBACK(s_apply_changes),
 							reinterpret_cast<gpointer>(this));
 
+	g_signal_connect(G_OBJECT(m_wSelectImageButton),
+							"clicked",
+							G_CALLBACK(s_select_image),
+							reinterpret_cast<gpointer>(this));
+
+	g_signal_connect(G_OBJECT(m_wNoImageButton),
+							"clicked",
+							G_CALLBACK(s_remove_image),
+							reinterpret_cast<gpointer>(this));
+
 	g_signal_connect(G_OBJECT(m_wCloseButton),
 							"clicked",
 							G_CALLBACK(s_close_window),
@@ -447,6 +576,11 @@ void AP_UnixDialog_FormatTable::_connectSignals(void)
 							"changed",
 							G_CALLBACK(s_apply_to_changed),
 							reinterpret_cast<gpointer>(this));
+
+	m_iBorderThicknessConnect = g_signal_connect(G_OBJECT(m_wBorderThickness),
+							"changed",
+							G_CALLBACK(s_border_thickness),
+							reinterpret_cast<gpointer>(this));
 }
 
 void AP_UnixDialog_FormatTable::_populateWindowData(void)
