@@ -461,7 +461,10 @@ bool AP_UnixFrame::_createViewGraphics(GR_Graphics *& pG, UT_uint32 iZoom)
 	AP_UnixFrameImpl * pImpl = static_cast<AP_UnixFrameImpl *>(getFrameImpl());
 	UT_ASSERT(pImpl);
 	UT_DEBUGMSG(("Got FrameImpl %x area %x \n",pImpl,pImpl->m_dArea));
-	pG = new GR_UnixGraphics(pImpl->m_dArea->window, fontManager, getApp());
+	//pG = new GR_UnixGraphics(pImpl->m_dArea->window, fontManager, getApp());
+	GR_UnixAllocInfo ai(pImpl->m_dArea->window, fontManager, getApp());
+	pG = (GR_UnixGraphics*) XAP_App::getApp()->newGraphics(ai);
+
 	ENSUREP_RF(pG);
 	pG->setZoomPercentage(iZoom);
 
