@@ -143,7 +143,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 
 	const XAP_StringSet* pSS = m_pApp->getStringSet();
 
-	UT_ASSERT(pSS);	// TODO: Would an error handler be more appropriate here?
+	UT_return_val_if_fail (pSS,1);	// TODO: Would an error handler be more appropriate here?
 
 	// Set the locale specific strings
 	struct control_id_string_id {
@@ -196,7 +196,7 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 	HWND hwndChild = GetDlgItem(hWnd, AP_RID_DIALOG_LIST_FRAME_PREVIEW);
 
 	// If we don't get the preview control things are bad!
-	UT_ASSERT(hwndChild);
+	UT_return_val_if_fail (hwndChild,1);
 
 	m_pPreviewWidget = new XAP_Win32PreviewWidget(static_cast<XAP_Win32App *>(m_pApp),
 													  hwndChild,
@@ -222,6 +222,18 @@ BOOL AP_Win32Dialog_Lists::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam
 	return 1;							// 1 == we did not call SetFocus()
 }
 
+
+void AP_Win32Dialog_Lists::setFoldLevelInGUI(void)
+{
+	UT_ASSERT_HARMLESS(0);
+}
+
+bool AP_Win32Dialog_Lists::isPageLists(void)
+{
+	UT_ASSERT_HARMLESS(0);
+	return true;
+}
+
 BOOL AP_Win32Dialog_Lists::_onCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
 	const WORD wNotifyCode	= HIWORD(wParam);
@@ -420,7 +432,7 @@ void AP_Win32Dialog_Lists::activate(void)
 
 	iResult = _win32Dialog.bringWindowToTop();
 
-	UT_ASSERT((iResult != 0));
+	UT_ASSERT_HARMLESS((iResult != 0));
 
 	_enableControls();
 }
@@ -446,20 +458,6 @@ void AP_Win32Dialog_Lists::notifyActiveFrame(XAP_Frame *pFrame)
 	_previewExposed();
 }
 
-void AP_Win32Dialog_Lists::notifyCloseFrame(XAP_Frame *pFrame)
-{
-	// Check that our hWnd still is valid. This is not always the case,
-	// such as when shutting down the application.
-	if (IsWindow(m_hThisDlg))
-	{
-		if (_win32Dialog.isParentFrame(*pFrame))
-		{
-			setbisCustomized(false);
-			setActiveFrame(getActiveFrame());
-		}
-		_updateCaption();
-	}
-}
 
 void AP_Win32Dialog_Lists::_enableControls(void)
 {
@@ -503,7 +501,7 @@ void AP_Win32Dialog_Lists::_onApply()
 	const bool bResumeList     = _isResumeListChecked();
 
 	// There can be only one.
-	UT_ASSERT((int(bStartNewList) + int(bApplyToCurrent) + int(bResumeList)) == 1);
+	UT_return_if_fail ((int(bStartNewList) + int(bApplyToCurrent) + int(bResumeList)) == 1);
 
 	setbStartNewList(bStartNewList);
 	setbApplyToCurrent(bApplyToCurrent);
@@ -582,7 +580,7 @@ void AP_Win32Dialog_Lists::_fillTypeList()
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
-	UT_ASSERT(pSS);	// TODO: Would an error handler be more appropriate here?
+	UT_return_if_fail (pSS);	// TODO: Would an error handler be more appropriate here?
 
 	static const XAP_String_Id rgIDs[] =
 	{
@@ -600,7 +598,7 @@ void AP_Win32Dialog_Lists::_fillTypeList()
 
 void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 {
-	UT_ASSERT(iType >= -1 && iType <=2);
+	UT_return_if_fail (iType >= -1 && iType <=2);
 
 	const UT_sint32 idComboStyle = AP_RID_DIALOG_LIST_COMBO_STYLE;
 
@@ -655,10 +653,10 @@ void AP_Win32Dialog_Lists::_fillStyleList(int iType)
 	}
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-	UT_ASSERT(pSS);	// TODO: Would an error handler be more appropriate here?
+	UT_return_if_fail (pSS);	// TODO: Would an error handler be more appropriate here?
 
 	HWND hComboStyle = GetDlgItem(m_hThisDlg, AP_RID_DIALOG_LIST_COMBO_STYLE);
-	UT_ASSERT(hComboStyle);
+	UT_return_if_fail (hComboStyle);
 	int nMaxWidth = 0;
 
 	// Get the HDC of the droplist to be able to get the
@@ -699,7 +697,7 @@ void AP_Win32Dialog_Lists::_typeChanged()
 {
 	const int iType = _getTypeComboCurSel();
 
-	const List_Type type =
+	const FL_ListType type =
 		iType == 1 ? BULLETED_LIST :
 		iType == 2 ? NUMBERED_LIST :
 		NOT_A_LIST;
@@ -776,7 +774,7 @@ void AP_Win32Dialog_Lists::_enableCustomControls(bool bEnable)
 void AP_Win32Dialog_Lists::_updateCaption()
 {
 	ConstructWindowName();
-	_win32Dialog.setDialogTitle((LPCSTR)(AP_Win32App::s_fromUTF8ToAnsi( getWindowName())).c_str());	
+	_win32Dialog.setDialogTitle((LPCSTR)(AP_Win32App::s_fromUTF8ToWinLocale( getWindowName())).c_str());	
 }
 
 void AP_Win32Dialog_Lists::_previewExposed()
@@ -899,10 +897,10 @@ void AP_Win32Dialog_Lists::_getDisplayedData(UT_sint32 controlId)
 }
 
 //
-// Returns the List_Type of the currently selected Type/Style combination.
+// Returns the FL_ListType of the currently selected Type/Style combination.
 // Returns NOT_A_LIST on failure.
 //
-List_Type AP_Win32Dialog_Lists::_getListTypeFromCombos() const
+FL_ListType AP_Win32Dialog_Lists::_getListTypeFromCombos() const
 {
 	const int iType  = _getTypeComboCurSel();
 	int iStyle = _getStyleComboCurSel();
@@ -916,19 +914,19 @@ List_Type AP_Win32Dialog_Lists::_getListTypeFromCombos() const
 
 	if (iType == 1 /* bullet list */)
 	{
-		return (List_Type)(iStyle + BULLETED_LIST);
+		return (FL_ListType)(iStyle + BULLETED_LIST);
 	}
 
 	if(iStyle >= BULLETED_LIST)
 		iStyle += (OTHER_NUMBERED_LISTS - BULLETED_LIST + 1);
 
 	// Numbered List, but just to make really REALLY sure, we assert it
-	UT_ASSERT(IS_NUMBERED_LIST_TYPE(iStyle));
+	UT_ASSERT_HARMLESS(IS_NUMBERED_LIST_TYPE(iStyle));
 
-	return (List_Type)iStyle;
+	return (FL_ListType)iStyle;
 }
 
-void AP_Win32Dialog_Lists::_setListType(List_Type type)
+void AP_Win32Dialog_Lists::_setListType(FL_ListType type)
 {
 	int iType;
 	int iStyle;
@@ -947,7 +945,7 @@ void AP_Win32Dialog_Lists::_setListType(List_Type type)
 	}
 	else
 	{
-		UT_ASSERT(IS_BULLETED_LIST_TYPE(type));
+		UT_ASSERT_HARMLESS(IS_BULLETED_LIST_TYPE(type));
 		iType = 1;
 		iStyle = type - BULLETED_LIST;
 	}
@@ -980,11 +978,11 @@ void AP_Win32Dialog_Lists::_selectFont()
 	}
 
 	XAP_DialogFactory* pDialogFactory = pFrame->getDialogFactory();
-	UT_ASSERT(pDialogFactory);
+	UT_return_if_fail (pDialogFactory);
 
 	XAP_Dialog_FontChooser* pDialog
 		= (XAP_Dialog_FontChooser *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_FONT));
-	UT_ASSERT(pDialog);
+	UT_ASSERT_HARMLESS(pDialog);
 
 	if (!pDialog)	// runtime failsafe
 	{
@@ -1092,14 +1090,14 @@ void AP_Win32Dialog_Lists::_selectFont()
 
 void AP_Win32Dialog_Lists::autoupdateLists(UT_Worker * pWorker)
 {
-	UT_ASSERT(pWorker);
+	UT_return_if_fail (pWorker);
 	// this is a static callback method and does not have a 'this' pointer.
 	AP_Win32Dialog_Lists* pDialog =
 		reinterpret_cast<AP_Win32Dialog_Lists*>(pWorker->getInstanceData());
 	// Handshaking code. Plus only update if something in the document
 	// changed.
 
-	UT_ASSERT(pDialog);
+	UT_ASSERT_HARMLESS(pDialog);
 
 	if (pDialog)	// runtime failsafe
 	{
