@@ -34,18 +34,71 @@
 
 /*****************************************************************/
 
-#define ENSUREP_C(p)		do { UT_ASSERT(p); if (!p) goto Cleanup; } while (0)
+#define ENSUREP_C(p)		do { UT_ASSERT_HARMLESS(p); if (!p) goto Cleanup; } while (0)
 
 /*****************************************************************/
 
 void AP_Frame::setZoomPercentage(UT_uint32 iZoom)
+{
+	XAP_Frame::setZoomPercentage(iZoom);
+	_showDocument(iZoom);
+}
+
+/*!
+ * This zooms an existing document in a frame by just changing the zoom
+ * in the graphics class.
+ */
+void AP_Frame::quickZoom(UT_uint32 iZoom)
 {
 	bool bChanged = (getZoomPercentage() != iZoom);
 	XAP_Frame::setZoomPercentage(iZoom);
-	if (bChanged) {
-		// only redisplay if zoom factor changed
-		_showDocument(iZoom);
+	FV_View * pView = static_cast<FV_View *>(getCurrentView());
+	if (bChanged) 
+	{
+		FL_DocLayout * pDocLayout = pView->getLayout();
+		pDocLayout->incrementGraphicTick();
+		GR_Graphics * pOldGraphics = pView->getGraphics();
+		pOldGraphics->setZoomPercentage(iZoom);
+		pOldGraphics->clearFont();
+		AP_TopRuler * pTop = pView->getTopRuler();
+		if(pTop)
+		{
+			pTop->setZoom(iZoom);
+		}
+		AP_LeftRuler * pLeft = pView->getLeftRuler();
+		if(pLeft)
+		{
+			pLeft->setZoom(iZoom);
+		}
+		setYScrollRange();
+		setXScrollRange();
+//
+// Redraw the entire screen
+//
+		pView->updateScreen(false);
+		pView->setPoint(pView->getPoint()); // place the cursor correctly
+		                                    // on the screen.
+		if(pTop && !pTop->isHidden())
+		{
+			pTop->draw(NULL);
+		}
+		if(pLeft && !pLeft->isHidden())
+		{
+			pLeft->draw(NULL);
+		}
 	}
+	else
+	{
+//
+// Redraw the entire screen
+//
+		pView->updateScreen(false);
+	}
+
+	// Notify listeners of new zoom (to update the zoom combo in toolbar)
+	// We do this regardless of bChanged since a change from 100% zoom to
+	// "Page Width" zoom may not change the logical zoom level.
+	pView->notifyListeners(AV_CHG_ALL);
 }
 
 UT_uint32 AP_Frame::getZoomPercentage(void)
@@ -60,7 +113,7 @@ AP_Frame::~AP_Frame()
 
 bool AP_Frame::initFrameData()
 {
-	UT_ASSERT(!static_cast<AP_FrameData*>(m_pData));
+	UT_ASSERT_HARMLESS(!static_cast<AP_FrameData*>(m_pData));
 
 	AP_FrameData* pData = new AP_FrameData(static_cast<XAP_App *>(m_pApp));
 
@@ -101,7 +154,7 @@ UT_Error AP_Frame::_loadDocument(const char * szFilename, IEFileType ieft,
 		m_pApp->rememberFrame(this);
 	}
 	AD_Document * pNewDoc = new PD_Document(getApp());
-	UT_ASSERT(pNewDoc);
+	UT_return_val_if_fail (pNewDoc, UT_ERROR);
 	
 	if (!szFilename || !*szFilename)
 	{
@@ -172,7 +225,7 @@ UT_Error AP_Frame::_importDocument(const char * szFilename, int ieft,
 	// if no filename, create a new document.
 
 	AD_Document * pNewDoc = new PD_Document(getApp());
-	UT_ASSERT(pNewDoc);
+	UT_return_val_if_fail (pNewDoc, UT_ERROR);
 
 	if (!szFilename || !*szFilename)
 	{
@@ -210,7 +263,9 @@ XAP_Frame * AP_Frame::buildFrame(XAP_Frame * pF)
 	if (!pClone->initialize())
 		goto Cleanup;
 
-
+	// we remember the view of the parent frame ...
+	static_cast<AP_FrameData*>(pClone->m_pData)->m_pRootView = m_pView;
+	
 	error = pClone->_showDocument(iZoom);
 	if (error)
 		goto Cleanup;
@@ -231,7 +286,7 @@ XAP_Frame * AP_Frame::buildFrame(XAP_Frame * pF)
 UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNew)
 {
 	bool bUpdateClones;
-	UT_Vector vClones;
+	UT_GenericVector<XAP_Frame*> vClones;
 	XAP_App * pApp = getApp();
 	UT_uint32 j = 0;
 	if(pApp->findFrame(this) < 0)
@@ -245,7 +300,7 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNe
 	}
 	for(j=0; j<vClones.getItemCount();j++)
 	{
-		XAP_Frame * pFrame = static_cast<XAP_Frame *>(vClones.getNthItem(j));
+		XAP_Frame * pFrame = vClones.getNthItem(j);
 		if(pApp->findFrame(pFrame) < 0)
 		{
 			pApp->rememberFrame(pFrame,this);
@@ -272,7 +327,7 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft, bool createNe
 	{
 		for (UT_uint32 i = 0; i < vClones.getItemCount(); i++)
 		{
-			AP_Frame * pFrame = static_cast<AP_Frame *>(vClones.getNthItem(i));
+			AP_Frame * pFrame = static_cast<AP_Frame*>(vClones.getNthItem(i));
 			if(pFrame != this)
 			{
 				pFrame->_replaceDocument(m_pDoc);
@@ -291,7 +346,7 @@ UT_Error AP_Frame::loadDocument(const char * szFilename, int ieft)
 UT_Error AP_Frame::importDocument(const char * szFilename, int ieft, bool markClean)
 {
 	bool bUpdateClones;
-	UT_Vector vClones;
+	UT_GenericVector<XAP_Frame*> vClones;
 	XAP_App * pApp = getApp();
 
 	bUpdateClones = (getViewNumber() > 0);
@@ -332,10 +387,10 @@ UT_Error AP_Frame::importDocument(const char * szFilename, int ieft, bool markCl
  */
 UT_uint32 AP_Frame::getNewZoom(XAP_Frame::tZoomType * tZoom)
 {
-	UT_Vector vecClones;
+	UT_GenericVector<XAP_Frame*> vecClones;
 	XAP_Frame *pF = NULL;
 	XAP_App * pApp = getApp();
-	UT_ASSERT(pApp);
+	UT_return_val_if_fail (pApp, 0);
 	XAP_Frame * pLastFrame = pApp->getLastFocussedFrame();
 	UT_uint32 iZoom = 100;
 	if(pLastFrame == NULL)
@@ -403,13 +458,13 @@ UT_Error AP_Frame::_showDocument(UT_uint32 iZoom)
 	if(isFrameLocked())
 	{
 		UT_DEBUGMSG(("_showDocument: Nasty race bug, please fix me!! \n"));
-		UT_ASSERT(0);
+		UT_ASSERT_HARMLESS(0);
 		return  UT_IE_ADDLISTENERERROR;
 	}
 	setFrameLocked(true);
 	if (!static_cast<AP_FrameData*>(m_pData))
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		setFrameLocked(false);
 		return UT_IE_IMPORTERROR;
 	}
@@ -523,7 +578,17 @@ void AP_Frame::_replaceView(GR_Graphics * pG, FL_DocLayout *pDocLayout,
 	PT_DocPosition inspt = 0;
 	FV_View * pOldView = NULL;
 	FL_DocLayout * pOldDocLayout  = NULL;
-	if (m_pView && !m_pView->isSelectionEmpty ()) {
+
+	// we want to remember point/selection when that is appropriate, which is when the new view is a
+	// view of the same doc as the old view, or if this frame is being cloned from an existing frame
+	
+	// these are the view and doc from which this frame is being cloned
+	FV_View * pRootView = NULL; // this should really be const, but
+								// getDocumentRangeOfCurrentSelection() is not
+	const AD_Document * pRootDoc = NULL;
+	
+	if (m_pView && !m_pView->isSelectionEmpty ())
+	{
 		holdsSelection = true;
 		static_cast<FV_View*>(m_pView)->getDocumentRangeOfCurrentSelection (&range);
 	} else if (m_pView)
@@ -531,6 +596,26 @@ void AP_Frame::_replaceView(GR_Graphics * pG, FL_DocLayout *pDocLayout,
 		inspt = static_cast<FV_View*>(m_pView)->getInsPoint ();
 		pOldView = static_cast<FV_View *>(m_pView);
 	}
+	else if(static_cast<AP_FrameData*>(m_pData)->m_pRootView)
+	{
+		pRootView = static_cast<FV_View*>(static_cast<AP_FrameData*>(m_pData)->m_pRootView);
+		pRootDoc = pRootView->getDocument();
+
+		if (!pRootView->isSelectionEmpty())
+		{
+			holdsSelection = true;
+			pRootView->getDocumentRangeOfCurrentSelection (&range);
+		} else if (pRootView)
+		{
+			inspt = pRootView->getInsPoint ();
+		}
+		else
+			hadView = false;
+
+		// we want to set m_pData->m_pRootView to NULL, since it has fullfilled its function and we
+		// do not want any dead pointers hanging around
+		static_cast<AP_FrameData*>(m_pData)->m_pRootView = NULL;
+	}
 	else
 		hadView = false;
 
@@ -543,15 +628,33 @@ void AP_Frame::_replaceView(GR_Graphics * pG, FL_DocLayout *pDocLayout,
 
 	REPLACEP(static_cast<AP_FrameData*>(m_pData)->m_pG, pG);
 	REPLACEP(static_cast<AP_FrameData*>(m_pData)->m_pDocLayout, pDocLayout);
+	
+	bool bSameDocument = false; // be cautious ...
 
-	if (pOldDoc != m_pDoc)
+	if(!pOldDoc)
+	{
+		// this is the case when this is a new frame unrelated to anything, or a frame cloned from
+		// existing frame
+		if(pRootDoc == m_pDoc)
+		{
+			// we have been cloned from a frame which related to the same document
+			bSameDocument = true;
+		}
+	}
+	else if (pOldDoc != m_pDoc)
 	{
 		UNREFP(pOldDoc);
 	}
+	else
+	{
+		// pOldDoc == m_pDoc
+		bSameDocument = true;
+	}
 
-	REPLACEP(m_pView, pView);
-	if(getApp()->getViewSelection() == m_pView)
-		getApp()->setViewSelection(NULL);
+	AV_View * pReplacedView = m_pView;
+	m_pView = pView;
+
+	getApp()->setViewSelection(NULL);
 
 	REPLACEP(m_pScrollObj, pScrollObj);
 	REPLACEP(m_pViewListener, pViewListener);
@@ -591,8 +694,19 @@ void AP_Frame::_replaceView(GR_Graphics * pG, FL_DocLayout *pDocLayout,
 
 	pDocLayout->fillLayouts();      
 
-	if (holdsSelection)
-		static_cast<FV_View*>(m_pView)->cmdSelect (range.m_pos1, range.m_pos2);
-	else if (hadView)
-	   static_cast<FV_View*>(m_pView)->moveInsPtTo(inspt);
+	// can only hold selection/point if the two views are for the same doc !!!
+	if(bSameDocument)
+	{
+		FV_View * pFV_View = static_cast<FV_View*>(m_pView);
+		if (holdsSelection)
+			pFV_View->cmdSelect (range.m_pos1, range.m_pos2);
+		else if (hadView)
+			pFV_View->moveInsPtTo(inspt);
+	}
+
+	if (XAP_FrameImpl * pFrameImpl = getFrameImpl())
+	{
+		pFrameImpl->notifyViewChanged(m_pView);
+	}
+	DELETEP(pReplacedView);
 }
