@@ -31,8 +31,7 @@
 #include "ap_Preview_Paragraph.h"
 #include "ap_Strings.h"
 #include "ap_Dialog_Lists.h"
-
-#include <fribidi.h>
+#include "gr_Painter.h"
 
 /************************************************************************/
 
@@ -49,10 +48,10 @@
 
 AP_Preview_Paragraph_Block::AP_Preview_Paragraph_Block(UT_RGBColor & clr,
 													   GR_Graphics * gc,
-													   AP_Dialog_Paragraph::tAlignState m_align,
+													   AP_Dialog_Paragraph::tAlignState align,
 													   UT_uint32 fontHeight)
 {
-	UT_ASSERT(gc);
+	UT_return_if_fail (gc);
 
 	m_clr.m_red = clr.m_red;
 	m_clr.m_grn = clr.m_grn;
@@ -68,7 +67,7 @@ AP_Preview_Paragraph_Block::AP_Preview_Paragraph_Block(UT_RGBColor & clr,
 	m_afterSpacing = DEFAULT_AFTER_SPACING;
 	m_lineSpacing = DEFAULT_LINE_SPACING;
 
-	m_align = AP_Dialog_Paragraph::align_LEFT;
+	m_align = align;
 	m_indent = AP_Dialog_Paragraph::indent_NONE;
 	m_spacing = AP_Dialog_Paragraph::spacing_SINGLE;
 
@@ -90,7 +89,7 @@ AP_Preview_Paragraph_Block::~AP_Preview_Paragraph_Block()
 
 void AP_Preview_Paragraph_Block::setText(const UT_UCSChar * text)
 {
-	UT_ASSERT(text);
+	UT_return_if_fail (text);
 
 	// clear the words vector by freeing ONLY the first
 	// word (see the destructor for notes)
@@ -213,7 +212,7 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 		m_firstLineLeftStop = m_leftStop;
 		break;
 	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+		UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 	}
 
 	if (lineSpacing)
@@ -221,6 +220,7 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 		m_spacing = spacing;
 		switch (m_spacing)
 		{
+		case AP_Dialog_Paragraph::spacing_UNDEF:
 		case AP_Dialog_Paragraph::spacing_SINGLE:
 			m_lineSpacing = 0;
 			break;
@@ -262,9 +262,9 @@ void AP_Preview_Paragraph_Block::setFormat(const XML_Char * pageLeftMargin,
 AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 										   const UT_UCSChar * text,
 										   AP_Dialog_Lists * dlg)
-  : XAP_Preview(gc), m_dir(FRIBIDI_TYPE_LTR)
+  : XAP_Preview(gc), m_dir(UT_BIDI_LTR)
 {
-	UT_ASSERT(text && dlg);
+	UT_ASSERT_HARMLESS(text && dlg);
 
 	m_font = NULL;
 	m_fontHeight = 0;
@@ -331,22 +331,25 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 	}
 
 	const XAP_StringSet * pSS = dlg->getApp()->getStringSet();
-
-	m_previousBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph).ucs4_str().ucs4_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph,s);
+	
+	m_previousBlock->setText(s.ucs4_str().ucs4_str());
 
 	// this text came from the current document, passed in as arg
 	m_activeBlock->setText(text);
 
-	m_followingBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph).ucs4_str().ucs4_str());
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph,s);
+	m_followingBlock->setText(s.ucs4_str().ucs4_str());
 }
 
 AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 					   const UT_UCSChar * text,
-										   XAP_Dialog * dlg) : XAP_Preview(gc), m_dir(FRIBIDI_TYPE_LTR)
+										   XAP_Dialog * dlg) : XAP_Preview(gc), m_dir(UT_BIDI_LTR)
 {
   // this method heavily relies upon the parent dlg to call setFormat()
   // rather than auto-generating defaults
-	UT_ASSERT(text && dlg);
+	UT_ASSERT_HARMLESS(text && dlg);
 
 	m_font = NULL;
 	m_fontHeight = 0;
@@ -385,21 +388,23 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 	}
 
 	const XAP_StringSet * pSS = dlg->getApp()->getStringSet();
-
-	m_previousBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph).ucs4_str().ucs4_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph,s);
+	m_previousBlock->setText(s.ucs4_str().ucs4_str());
 
 	// this text came from the current document, passed in as arg
 	m_activeBlock->setText(text);
-
-	m_followingBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph).ucs4_str().ucs4_str());
+	
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph,s);
+	m_followingBlock->setText(s.ucs4_str().ucs4_str());
 }
 
 AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 					   const UT_UCSChar * text,
 					   AP_Dialog_Paragraph * dlg)
-	: XAP_Preview(gc),m_dir(FRIBIDI_TYPE_LTR)
+	: XAP_Preview(gc),m_dir(UT_BIDI_LTR)
 {
-	UT_ASSERT(text && dlg);
+	UT_ASSERT_HARMLESS(text && dlg);
 
 	m_font = NULL;
 	m_fontHeight = 0;
@@ -448,7 +453,7 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 									(AP_Dialog_Paragraph::tSpacingState) dlg->_getMenuItemValue(AP_Dialog_Paragraph::id_MENU_SPECIAL_SPACING));
 
 		if(dlg->_getCheckItemValue(AP_Dialog_Paragraph::id_CHECK_DOMDIRECTION) == AP_Dialog_Paragraph::check_TRUE)
-			m_dir = FRIBIDI_TYPE_RTL;
+			m_dir = UT_BIDI_RTL;
 	}
 
 	{
@@ -467,19 +472,19 @@ AP_Preview_Paragraph::AP_Preview_Paragraph(GR_Graphics * gc,
 	}
 
 	const XAP_StringSet * pSS = dlg->m_pApp->getStringSet();
-
-	m_previousBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph).ucs4_str().ucs4_str());
+	UT_UTF8String s;
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewPrevParagraph,s);
+	m_previousBlock->setText(s.ucs4_str().ucs4_str());
 
 	// this text came from the current document, passed in as arg
 	m_activeBlock->setText(text);
-
-	m_followingBlock->setText(pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph).ucs4_str().ucs4_str());
+	
+	pSS->getValueUTF8(AP_STRING_ID_DLG_Para_PreviewFollowParagraph,s);
+	m_followingBlock->setText(s.ucs4_str().ucs4_str());
 }
 
 AP_Preview_Paragraph::~AP_Preview_Paragraph()
 {
-	DELETEP(m_font);
-
 	DELETEP(m_clrWhite);
 	DELETEP(m_clrBlack);
 	DELETEP(m_clrGray);
@@ -501,7 +506,7 @@ void AP_Preview_Paragraph::setFormat(const XML_Char * pageLeftMargin,
 										const XML_Char * lineSpacing,
 										AP_Dialog_Paragraph::tSpacingState spacing)
 {
-	UT_ASSERT(m_activeBlock);
+	UT_return_if_fail(m_activeBlock);
 	m_activeBlock->setFormat(pageLeftMargin, pageRightMargin,
 								align, firstLineIndent, indent, leftIndent,
 								rightIndent, beforeSpacing, afterSpacing,
@@ -510,7 +515,7 @@ void AP_Preview_Paragraph::setFormat(const XML_Char * pageLeftMargin,
 
 void AP_Preview_Paragraph::draw(void)
 {
-	UT_ASSERT(m_gc);
+	UT_return_if_fail (m_gc);
 
 	// paint white background (Word 97's background is always white, the
 	// text is always black, and the font is always the same size, regardless
@@ -534,7 +539,7 @@ bool AP_Preview_Paragraph::_loadDrawFont(void)
 
 	if (font)
 	{
-		REPLACEP(m_font, font);
+		m_font = font;
 		m_gc->setFont(m_font);
 		m_fontHeight = m_gc->getFontHeight();
 		return true;
@@ -546,23 +551,26 @@ bool AP_Preview_Paragraph::_loadDrawFont(void)
 void AP_Preview_Paragraph::_drawPageBackground(void)
 {
 	// clear area
-	m_gc->fillRect(*m_clrWhite, 0, 0, m_gc->tlu(getWindowWidth()), m_gc->tlu(getWindowHeight()));
+	GR_Painter painter(m_gc);
+	painter.fillRect(*m_clrWhite, 0, 0, m_gc->tlu(getWindowWidth()), m_gc->tlu(getWindowHeight()));
 }
 
 void AP_Preview_Paragraph::_drawPageBorder(void)
 {
+	GR_Painter painter(m_gc);
+
 	// draw a black one pixel border
 	m_gc->setColor(*m_clrBlack);
-	m_gc->drawLine(0, 0, m_gc->tlu(getWindowWidth()), 0);
-	m_gc->drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()));
-	m_gc->drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1));
-	m_gc->drawLine(0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, 0);
+	painter.drawLine(0, 0, m_gc->tlu(getWindowWidth()), 0);
+	painter.drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()));
+	painter.drawLine(m_gc->tlu(getWindowWidth()) - m_gc->tlu(1), m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1));
+	painter.drawLine(0, m_gc->tlu(getWindowHeight()) - m_gc->tlu(1), 0, 0);
 }
 
 
 void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 {
-	UT_ASSERT(block);
+	UT_return_if_fail (block);
 
 	UT_uint32 ypre = 0;
 	UT_uint32 ypost = 0;
@@ -577,6 +585,7 @@ void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 		// we have to land the baseline in the right spot then
 		switch (block->m_spacing)
 		{
+		case AP_Dialog_Paragraph::spacing_UNDEF:
 		case AP_Dialog_Paragraph::spacing_SINGLE:
 		case AP_Dialog_Paragraph::spacing_ONEANDHALF:
 		case AP_Dialog_Paragraph::spacing_DOUBLE:
@@ -588,7 +597,7 @@ void AP_Preview_Paragraph::_appendBlock(AP_Preview_Paragraph_Block * block)
 			ypre = block->m_lineSpacing;
 			break;
 		default:
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+			UT_ASSERT_HARMLESS(UT_SHOULD_NOT_HAPPEN);
 		}
 	}
 
@@ -651,7 +660,7 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 											UT_uint32 y)
 {
 
-	UT_ASSERT(words && widths);
+	UT_return_val_if_fail (words && widths, 0);
 
 	// width of space character in pixels
 	UT_sint32 spaceCharWidth = m_gc->tlu(3);
@@ -659,7 +668,7 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 	UT_uint32 i = 0;
 	UT_uint32 totalWords = words->getItemCount();
 
-	UT_sint32 pixelsForThisLine = 0;
+	UT_uint32 pixelsForThisLine = 0;
 
 	// max length of first line is the diff between the (special)
 	// left stop and the (normal) right stop
@@ -695,7 +704,7 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 
 	UT_uint32 willDrawAt = left;
 
-	if(m_dir == FRIBIDI_TYPE_RTL)
+	if(m_dir == UT_BIDI_RTL)
 		willDrawAt += maxPixelsForThisLine;
 
  	spaceCharWidth <<= 8;	// Calculate spacing at 256 times the resolution
@@ -704,7 +713,7 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
  	switch(align)
  	{
  	case AP_Dialog_Paragraph::align_RIGHT:
-		if(m_dir == FRIBIDI_TYPE_LTR)
+		if(m_dir == UT_BIDI_LTR)
 
 	    // for right, we just draw at the difference in spaces added onto the first line stop.
 	    willDrawAt = left + (maxPixelsForThisLine - pixelsForThisLine);
@@ -722,7 +731,7 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 		break;
 	default:
 		// aligh_LEFT is caught here
-		if(m_dir == FRIBIDI_TYPE_RTL)
+		if(m_dir == UT_BIDI_RTL)
 			willDrawAt =  pixelsForThisLine + left;
 
 		break;
@@ -732,39 +741,31 @@ UT_uint32 AP_Preview_Paragraph::_appendLine(UT_Vector * words,
 
 	UT_uint32 k;
 
-	FriBidiChar fb1[100];
-	FriBidiChar fb2[100];
-	UT_UCSChar  str[100];
-	UT_uint32 j, iLen;
+	GR_Painter painter(m_gc);
 
+	UT_UCS4String s;
+	UT_UCS4Char *pBuf = NULL;
+	UT_uint32 size = 0;
 	for (k = startWithWord; k < i; k++)
 	{
 		// this will not produce correct results in true bidi text, since the words that are inconsistend
 		// with the overall pargraph direction will be in wrong order, but that is not a big deal
-  	    iLen = UT_UCS4_strlen((UT_UCSChar *) words->getNthItem(k));
-	    for(j = 0; j < iLen; j++)
-		    fb1[j] = ((UT_UCSChar *) words->getNthItem(k))[j];
+		s = (UT_UCS4Char *)words->getNthItem(k);
+		size = s.size() + 1;
+		pBuf = (UT_UCS4Char *)UT_calloc(size, sizeof(UT_UCS4Char));
+		memset(pBuf, 0, size * sizeof(UT_UCS4Char));
 
-		fribidi_log2vis (		/* input */
-						 &fb1[0],
-						 iLen,
-						 &m_dir,
-						 /* output */
-						 &fb2[0],
-						 NULL,
-						 NULL,
-						 NULL);
+		UT_bidiReorderString(s.ucs4_str(), s.size(), m_dir, pBuf);
 
-		for(j = 0; j < iLen; j++)
-		    str[j] = (UT_UCSChar)fb2[j];
-
-		if(m_dir == FRIBIDI_TYPE_RTL)
+		if(m_dir == UT_BIDI_RTL)
 		    willDrawAt -= ((widths->getNthItem(k)) << 8) + spaceCharWidth;
 
-		m_gc->drawChars(str, 0,	iLen, willDrawAt >> 8, y);
+		painter.drawChars(pBuf, 0, s.size(), willDrawAt >> 8, y);
 
-		if(m_dir == FRIBIDI_TYPE_LTR)
+		if(m_dir == UT_BIDI_LTR)
 		    willDrawAt += ((widths->getNthItem(k)) << 8) + spaceCharWidth;
+
+		FREEP(pBuf);
 	}
 
 	// return number of words drawn
