@@ -25,10 +25,10 @@
 #include "gr_Graphics.h"
 #include "ap_Ruler.h"
 
-#define SCALE_IN 1
-#define SCALE_CM 1
-#define SCALE_MM 1
-#define SCALE_PT 1
+#define SCALE_IN 10
+#define SCALE_CM 10
+#define SCALE_MM 10
+#define SCALE_PT 10
 
 ap_RulerTicks::ap_RulerTicks(GR_Graphics * pG, UT_Dimension dim)
 {
@@ -78,7 +78,7 @@ ap_RulerTicks::ap_RulerTicks(GR_Graphics * pG, UT_Dimension dim)
 		tickUnit = UT_convertToLogicalUnits(Buffer);
 		tickLong = 2;
 		tickLabel = 4;
-		tickScale = 1;
+		tickScale = 10;
 		sprintf(Buffer, "%fmm", (dBasicUnit / 2 * (double)tickUnitScale));
 		dragDelta = UT_convertToLogicalUnits(Buffer);
 		break;
@@ -108,7 +108,7 @@ ap_RulerTicks::ap_RulerTicks(GR_Graphics * pG, UT_Dimension dim)
 		break;
 
 	default:
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		UT_ASSERT_HARMLESS(UT_NOT_IMPLEMENTED);
 		break;
 	}
 }
