@@ -45,6 +45,7 @@ toolbariconmap(EDIT_REMOVEFOOTER,tb_edit_removefooter_xpm)
 toolbariconmap(SPELLCHECK,tb_spellcheck_xpm)
 toolbariconmap(TRANSPARENTLANG,tb_transparent_xpm)
 toolbariconmap(IMG,tb_insert_graphic_xpm)
+toolbariconmap(IMG,tb_remove_graphic_xpm)
 toolbariconmap(FMT_STYLE,NoIcon)
 toolbariconmap(FMT_FONT,NoIcon)
 toolbariconmap(FMT_HYPERLINK,tb_hyperlink_xpm)
@@ -97,6 +98,12 @@ toolbariconmap(MERGELEFT, tb_MergeLeft_xpm)
 toolbariconmap(MERGERIGHT, tb_MergeRight_xpm)
 toolbariconmap(MERGEABOVE, tb_MergeAbove_xpm)
 toolbariconmap(MERGEBELOW, tb_MergeBelow_xpm)
+toolbariconmap(SPLITLEFT, tb_SplitLeft_xpm)
+toolbariconmap(SPLITHORIMID, tb_SplitHoriMid_xpm)
+toolbariconmap(SPLITRIGHT, tb_SplitRight_xpm)
+toolbariconmap(SPLITABOVE, tb_SplitAbove_xpm)
+toolbariconmap(SPLITVERTMID, tb_SplitVertMid_xpm)
+toolbariconmap(SPLITBELOW, tb_SplitBelow_xpm)
 // Format Table DlgBox
 toolbariconmap(FT_LINEBOTTOM, tb_LineBottom_xpm)
 toolbariconmap(FT_LINETOP, tb_LineTop_xpm)
