@@ -1,6 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 4; indent-tabs-mode: t -*- */
 /* Copyright (C) 1999 AbiSource, Inc.
  * Copyright (C) 2003 Tomas Frydrych <tomas@frydrych.uklinux.net>
+ * Copyright (C) 2004 Hubert Figuiere <hfiguiere@teaser.fr>
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,18 +32,21 @@
 #include "ut_stack.h"
 #include "pt_Types.h"
 #include "pd_Document.h"
+#include "pp_Revision.h"
 #include "ut_mbtowc.h"
 #include "fl_AutoLists.h"
 #include "fl_AutoNum.h"
 #include "fl_BlockLayout.h"
-#include <fribidi.h>
 #include  "ie_Table.h"
+#include "ie_imp_RTFParse.h"
 
 class IE_Imp_RTF;
 class RTF_msword97_list;
 class ie_imp_cell;
 class ie_imp_table;
 class ie_imp_table_control;
+class RTFProps_FrameProps;
+
 
 // Font table entry
 struct ABI_EXPORT RTFFontTableItem
@@ -92,8 +96,11 @@ public:
 	UT_sint32  m_styleNumber ; //index into the style table
 	UT_uint32  m_listTag; // tag for lists to hang off
 	const char * m_szLang;
-	bool    m_RTL;
-	FriBidiCharType m_dirOverride;
+	UT_BidiCharType m_dir;
+	UT_BidiCharType m_dirOverride;
+	bool    m_Hidden;
+	PP_RevisionType m_eRevision;
+	UT_uint32 m_iCurrentRevisionId;
 };
 
 class ABI_EXPORT RTFProps_bCharProps
@@ -123,6 +130,7 @@ public:
 	bool bm_listTag; // tag for lists to hanfg off
 	bool bm_RTL;
 	bool bm_dirOverride;
+	bool bm_Hidden;
 };
 
 struct ABI_EXPORT _rtfListTable
@@ -133,7 +141,7 @@ struct ABI_EXPORT _rtfListTable
 	bool simple;
 	bool continueList;
 	bool hangingIndent;
-	List_Type type;
+	FL_ListType type;
 	bool bold;
 	bool italic;
 	bool caps;
@@ -215,9 +223,11 @@ struct ABI_EXPORT RTFProps_ParaProps
 	UT_uint32       m_iOverrideLevel;     // 0's index to the level
 	_rtfListTable   m_rtfListTable;
 	UT_sint32  m_styleNumber ; //index into the style table
-	bool            m_RTL;
+	UT_BidiCharType m_dir;
 	UT_sint32       m_tableLevel; //nesting level of the paragram in a table.
 	bool            m_bInTable; // true if paragraph is in a table
+	PP_RevisionType m_eRevision;
+	UT_uint32 m_iCurrentRevisionId;
 };
 
 // These are set true if changed in list definitions.
@@ -371,12 +381,17 @@ struct ABI_EXPORT RTFProps_ImageProps
 
 	RTFProps_ImageProps ();
 	IPSizeType sizeType;
-	UT_uint32 wGoal;
-	UT_uint32 hGoal;
+	UT_uint16 wGoal;
+	UT_uint16 hGoal;
 	UT_uint16 scaleX;
 	UT_uint16 scaleY;
 	UT_uint32 width;
 	UT_uint32 height;
+	bool bCrop;
+	UT_sint32 cropt;
+	UT_sint32 cropb;
+	UT_sint32 cropl;
+	UT_sint32 cropr;
 };
 
 // Section properties
@@ -399,7 +414,7 @@ struct ABI_EXPORT RTFProps_SectionProps
 	UT_sint32       m_footerYTwips;
 	UT_sint32       m_gutterTwips;
 	UT_sint32       m_colSpaceTwips;
-	FriBidiCharType m_dir;
+	UT_BidiCharType m_dir;
 };
 
 
@@ -432,7 +447,7 @@ struct ABI_EXPORT RTFHdrFtr
 struct ABI_EXPORT RTFStateStore
 {
 	RTFStateStore();
-
+    RTFStateStore * clone();
 //	RTFStateStore& operator=(const RTFStateStore&);
 
 	enum DestinationStateTypes { rdsNorm, rdsSkip, rdsFootnote, rdsHeader, rdsFooter, rdsField };
@@ -445,7 +460,6 @@ struct ABI_EXPORT RTFStateStore
 	RTFProps_SectionProps	m_sectionProps;			// Section properties
 	RTFProps_CellProps      m_cellProps;            // Cell properties
     RTFProps_TableProps     m_tableProps;           // Table properties
-
 	UT_uint32				m_unicodeAlternateSkipCount;	// value of N in "\ucN"
 	UT_uint32				m_unicodeInAlternate;			// chars left in alternate "\u<u><A>"
 };
@@ -516,11 +530,16 @@ public:
 
 	virtual bool supportsLoadStylesOnly() {return true;}
 	virtual UT_Error	importFile(const char * szFilename);
-	virtual void		pasteFromBuffer(PD_DocumentRange * pDocRange,
-										const unsigned char * pData, UT_uint32 lenData, const char * szEncoding = 0);
-	UT_sint32 get_vecWord97ListsCount(void) { return m_vecWord97Lists.getItemCount();}
-	RTF_msword97_list *  get_vecWord97NthList(UT_sint32 i) { return static_cast<RTF_msword97_list *>(m_vecWord97Lists.getNthItem(i));}
-    bool  isWord97Lists(void) const { return (m_vecWord97Lists.getItemCount() > 0);}
+	virtual bool		pasteFromBuffer(PD_DocumentRange * pDocRange,
+										const unsigned char * pData, 
+                                        UT_uint32 lenData, 
+                                        const char * szEncoding = 0);
+	UT_sint32 get_vecWord97ListsCount(void) 
+        { return m_vecWord97Lists.getItemCount();}
+	RTF_msword97_list *  get_vecWord97NthList(UT_sint32 i) 
+        { return m_vecWord97Lists.getNthItem(i);}
+    bool  isWord97Lists(void) const 
+        { return (m_vecWord97Lists.getItemCount() > 0);}
 
 	enum PictFormat {
 		picNone,
@@ -550,29 +569,62 @@ private:
 	} RTFBookmarkType;
 
 	bool AddChar(UT_UCSChar ch);
+public:
 	bool FlushStoredChars(bool forceInsertPara = false);
+private:
 	bool StartNewPara();
+	bool HandleParKeyword();
 	bool StartNewSection();
+public:
 	bool PushRTFState();
 	bool PopRTFState();
-	bool ParseRTFKeyword();
 	bool ParseChar(UT_UCSChar ch,bool no_convert=1);
+	void setStruxImage(bool b)
+		{ m_bStruxImage = b;}
+	bool isStruxImage(void) const
+		{ return m_bStruxImage;}
+private:
+	bool ParseRTFKeyword();
 	bool ReadCharFromFileWithCRLF(unsigned char* pCh);
 	bool ReadCharFromFile(unsigned char* pCh);
 	UT_UCS4Char ReadHexChar(void);
 	bool SkipBackChar(unsigned char ch);
-	bool ReadKeyword(unsigned char* pKeyword, UT_sint16* pParam, bool* pParamUsed,
+	bool ReadKeyword(unsigned char* pKeyword, UT_sint32* pParam, 
+                     bool* pParamUsed,
 					 UT_uint32 keywordBuffLen);
-	bool TranslateKeyword(unsigned char* pKeyword, UT_sint16 param, bool fParam);
+	bool TranslateKeyword(unsigned char* pKeyword, UT_sint32 param, 
+                          bool fParam);
+public:
+	bool TranslateKeywordID(RTF_KEYWORD_ID keywordID, 
+							UT_sint32 param, bool fParam);
+	bool insertStrux(PTStruxType pts , const XML_Char ** attrs=NULL, const XML_Char ** props=NULL);
+
+	/*  Parser stuff */
+	bool StandardKeywordParser(IE_Imp_RTFGroupParser *parser);
+private:
+	RTF_KEYWORD_ID KeywordToID(const char * keyword);
+
+	bool HandleStarKeyword();
+
+	bool HandlePCData(UT_UTF8String & str);
+
 	bool ReadColourTable();
 	bool ReadFontTable();
-	bool ReadOneFontFromTable();
+	bool ReadOneFontFromTable(bool bNested);
+	bool ReadRevisionTable();
+public:
 	bool HandlePicture();
+	void clearImageName(void)
+		{ m_sImageName.clear();}
+
+private:
 	bool HandleObject();
 	bool HandleField();
 	bool HandleStyleDefinition(void);
 	bool HandleHeaderFooter(RTFHdrFtr::HdrFtrType hftype, UT_uint32 & headerID);
+public:
 	bool SkipCurrentGroup(bool bConsumeLastBrace = false);
+private:
 	bool StuffCurrentGroup(UT_ByteBuf & buf);
 	bool LoadPictData(PictFormat format, const char * image_name,
 					  struct RTFProps_ImageProps & imgProps,
@@ -589,7 +641,12 @@ private:
 	bool HandleListLevel(RTF_msword97_list * pList, UT_uint32 levelCount  );
 	bool HandleTableList(void);
 	char * getCharsInsideBrace(void);
-	bool ParseCharParaProps( unsigned char * pKeyword, UT_sint16 param, bool fParam, RTFProps_CharProps * pChars, RTFProps_ParaProps * pParas, RTFProps_bCharProps * pbChars, RTFProps_bParaProps * pbParas);
+	bool ParseCharParaProps(unsigned char * pKeyword, 
+                            UT_sint32 param, bool fParam, 
+                            RTFProps_CharProps * pChars, 
+                            RTFProps_ParaProps * pParas, 
+                            RTFProps_bCharProps * pbChars, 
+                            RTFProps_bParaProps * pbParas);
 	bool ReadListOverrideTable(void);
 	bool HandleTableListOverride(void);
 
@@ -607,6 +664,7 @@ private:
 	bool HandleDeleted(bool state);
 	bool HandleBold(bool state);
 	bool HandleItalic(bool state);
+	bool HandleHidden(bool state);
 	bool HandleUnderline(bool state);
 	bool HandleOverline(bool state);
 	bool HandleStrikeout(bool state);
@@ -620,6 +678,9 @@ private:
 	bool HandleBookmark (RTFBookmarkType type);
 	bool HandleListTag(long id);
 
+	bool HandleRevisedText(PP_RevisionType eType, UT_uint32 iId);
+	bool HandleRevisedTextTimestamp(UT_uint32 iDttm);
+
 	// Generic handlers
 	bool HandleFloatCharacterProp(double val, double* pProp);
 	bool HandleU32CharacterProp(UT_uint32 val, UT_uint32* pProp);
@@ -629,12 +690,14 @@ private:
 
 	// Paragraph property handlers
 	bool ResetParagraphAttributes();
-	bool ApplyParagraphAttributes();
+	bool ApplyParagraphAttributes(bool bDontInsert = false);
 	bool SetParaJustification(RTFProps_ParaProps::ParaJustification just);
 	bool AddTabstop(UT_sint32 stopDist, eTabType tabType, eTabLeader tableader);
-	bool AddTabstop(UT_sint32 stopDist, eTabType tabType, eTabLeader tabLeader,  RTFProps_ParaProps * pParas);
+	bool AddTabstop(UT_sint32 stopDist, eTabType tabType, 
+                    eTabLeader tabLeader,  RTFProps_ParaProps * pParas);
 
 // Paste AbiWord tables
+ public:
 	bool HandleAbiTable(void);
 	bool HandleAbiCell(void);
 	bool HandleAbiEndTable(void);
@@ -662,11 +725,32 @@ private:
     void           HandleCell(void);
 	void           HandleCellX(UT_sint32 cellx);
     void           HandleRow(void);
+	UT_sint32      getPasteDepth(void);
+	void           closePastedTableIfNeeded(void);
+
+ private:
+
 	void           HandleNote();
 	void           HandleNoteReference();
+// Shape handlers in ie_imp_RTFObjectsAndPicts.cpp
+	void           HandleShape(void);
+	void           HandleShapePict(void);
+
+public:
+	void           HandleShapeText(RTFProps_FrameProps & frame);
+	void           addFrame(RTFProps_FrameProps & frame);
+	bool           isFrameIn(void)
+		{ return m_bFrameStruxIn;}
+
 	bool           bUseInsertNotAppend(void);
+
+private:
+//	void           HandleEndShape(void);
+// Meta data
+	bool           HandleInfoMetaData(void);
 // Little convience wrapper
-	void           _setStringProperty(UT_String & sPropString, const char * szProp, const char * szVal);
+	void           _setStringProperty(UT_String & sPropString, 
+                                      const char * szProp, const char * szVal);
 
 	// Section property handlers
 	bool ApplySectionAttributes();
@@ -679,16 +763,28 @@ private:
 	    RTF_TOKEN_DATA,
 	    RTF_TOKEN_ERROR = -1
 	} RTFTokenType;
-	RTFTokenType NextToken (unsigned char *pKeyword, UT_sint16* pParam,
-							bool* pParamUsed, UT_uint32 len, bool bIgnoreWhiteSpace=false);
+	RTFTokenType NextToken (unsigned char *pKeyword, UT_sint32* pParam,
+							bool* pParamUsed, UT_uint32 len, 
+                            bool bIgnoreWhiteSpace = false);
 
 	UT_Error _isBidiDocument();
 	bool     _appendSpan();
 	bool     _insertSpan();
+	void     _formRevisionAttr(UT_String & s,UT_String & props, const XML_Char * style);
+	
+
+private:
+	// static helpers to decode
+	static bool digVal(char ch, int& value, int base);
+	static bool hexVal(char c, int& value);
 	
 
 // import member vars
 private:
+	/* keyword for new gen parser */
+	static bool keywordSorted;
+	static void _initialKeywordSort(void);
+
 	UT_GrowBuf m_gbBlock;
 	char *m_szFileDirName;
 
@@ -715,10 +811,10 @@ private:
 	UT_Stack m_stateStack;
 	RTFStateStore m_currentRTFState;
 
-	UT_Vector m_fontTable;
+	UT_GenericVector<RTFFontTableItem*> m_fontTable;
 	UT_NumberVector m_colourTable;
 	UT_Vector m_hdrFtrTable;
-	UT_Vector m_styleTable;
+	UT_GenericVector<XML_Char *> m_styleTable;
 
 	struct _rtfAbiListTable
 	{
@@ -730,8 +826,9 @@ private:
 		UT_uint32 mapped_id;
 		UT_uint32 mapped_parentid;
 	};
-	UT_Vector m_vecAbiListTable;
-	_rtfAbiListTable * getAbiList( UT_uint32 i) {return static_cast<_rtfAbiListTable *>(m_vecAbiListTable.getNthItem(i));}
+	UT_GenericVector<_rtfAbiListTable *> m_vecAbiListTable;
+	_rtfAbiListTable * getAbiList( UT_uint32 i) 
+        {return m_vecAbiListTable.getNthItem(i);}
 
 	RTF_msword97_listOverride* _getTableListOverride(UT_uint32 id);
 
@@ -747,16 +844,19 @@ private:
 	UT_uint32 			m_lenPasteBuffer;
 	const unsigned char *		m_pCurrentCharInPasteBuffer;
 	PT_DocPosition		m_dposPaste;
+	PT_DocPosition		m_dOrigPos;
 	UT_uint32		    deflangid;
 	UT_UCS4_mbtowc		m_mbtowc;
 	bool                m_parsingHdrFtr;
 	UT_uint32           m_icurOverride;
 	UT_uint32           m_icurOverrideLevel;
-	UT_Vector           m_vecWord97Lists;
-	UT_Vector           m_vecWord97ListOverride;
+	UT_GenericVector<RTF_msword97_list *> m_vecWord97Lists;
+	UT_GenericVector<RTF_msword97_listOverride*> m_vecWord97ListOverride;
 	void _appendHdrFtr ();
-	bool _appendField (const XML_Char *xmlField, const XML_Char ** pszAttribs=NULL);
-	XML_Char *_parseFldinstBlock (UT_ByteBuf & buf, XML_Char *xmlField, bool & isXML);
+	bool _appendField (const XML_Char *xmlField, 
+                       const XML_Char ** pszAttribs=NULL);
+	XML_Char *_parseFldinstBlock (UT_ByteBuf & buf, XML_Char *xmlField, 
+                                  bool & isXML);
 	bool                m_bAppendAnyway;
 	RTFProps_SectionProps m_sectdProps ;
 	ie_imp_table_control  m_TableControl;
@@ -785,16 +885,17 @@ private:
 	RTFStateStore         m_FootnoteRefState;
 	bool                  m_bFieldRecognized;
 	UT_sint32             m_iIsInHeaderFooter;
+	bool                  m_bSectionHasPara;
+	bool                  m_bStruxInserted;
+	bool		          m_bStruxImage;
+	UT_UTF8String	      m_sImageName;
+	bool                  m_bFrameStruxIn;
+
+	UT_BidiCharType       m_iAutoBidiOverride;
+	UT_BidiCharType       m_iBidiLastType;
+	UT_BidiCharType       m_iBidiNextType;
+
 };
 
 #endif /* IE_IMP_RTF_H */
 
-
-
-
-
-
-
-
-
-
