@@ -523,7 +523,8 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 
 	int listID = 0, startingNumber = 0, level = 1;
 	char listType = '1';
-	WPXString textBeforeNumber, textAfterNumber;
+	UT_UTF8String textBeforeNumber, textAfterNumber;
+	
 	if (propList["libwpd:id"])
 		listID = propList["libwpd:id"]->getInt();
 	if (propList["text:start-value"])
@@ -531,9 +532,9 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 	if (propList["libwpd:level"])
 		level = propList["libwpd:level"]->getInt();
 	if (propList["style:num-prefix"])
-		textBeforeNumber = propList["style:num-prefix"]->getStr();
+		textBeforeNumber += propList["style:num-prefix"]->getStr().cstr();
 	if (propList["style:num-suffix"])
-		textAfterNumber = propList["style:num-suffix"]->getStr();
+		textAfterNumber += propList["style:num-suffix"]->getStr().cstr();
 	if (propList["style:num-format"])
 	{
 		listType = propList["style:num-format"]->getStr().cstr()[0];
@@ -547,7 +548,7 @@ void IE_Imp_WordPerfect::defineOrderedListLevel(const WPXPropertyList &propList)
 		if (m_pCurrentListDefinition)
 			delete (m_pCurrentListDefinition);
 
-		fprintf(stderr, "WLACH: Outline hash: %i\n");
+		fprintf(stderr, "WLACH: Outline hash - listID: %i\n", listID);
 		m_pCurrentListDefinition = new ABI_ListDefinition(listID);
 	}
 	
@@ -964,8 +965,8 @@ UT_Error IE_Imp_WordPerfect::_appendSection(int numColumns, const float marginLe
 // NB: AbiWord-2.0 doesn't properly support setting list delimeters at levels greater than 1,
 // we hack around this by using only "plain" (e.g.: NULL) list delimeters on levels greater than 1.
 UT_Error IE_Imp_WordPerfect::_updateDocumentOrderedListDefinition(ABI_ListDefinition *pListDefinition, int iLevel, 
-																  const char listType, const WPXString &sTextBeforeNumber, 
-																  const WPXString &sTextAfterNumber, int iStartingNumber)
+																  const char listType, const UT_UTF8String &sTextBeforeNumber, 
+																  const UT_UTF8String &sTextAfterNumber, int iStartingNumber)
 {
 	UT_DEBUGMSG(("AbiWordPerfect: Updating document list definition (iLevel: %i)\n", iLevel));
 
@@ -991,11 +992,12 @@ UT_Error IE_Imp_WordPerfect::_updateDocumentOrderedListDefinition(ABI_ListDefini
 		}   
 		else 
 		{
-			UT_UCS4String sNumberingString;
-			UT_UCS4String sNumber("%L", 0, false);
-			sNumberingString += sTextBeforeNumber.cstr();
+			UT_UTF8String sNumberingString;
+			UT_UTF8String sNumber("%L", (size_t)0);
+			
+			sNumberingString += sTextBeforeNumber;
 			sNumberingString += sNumber;
-			sNumberingString += sTextAfterNumber.cstr();
+			sNumberingString += sTextAfterNumber;
 	
 			pAuto = new fl_AutoNum(pListDefinition->getListID(iLevel), 0, pListDefinition->getListType(iLevel), iStartingNumber, 
 								   const_cast<XML_Char*>(reinterpret_cast<const XML_Char*>(sNumberingString.utf8_str())), ".", getDoc(), NULL);
